/** Java class "Affinity_3D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;
/**
 * <p>
 * This class contains constructors to build a Affinity_3D. Its matrix needs 12
 * entries to build it:
 * A Affinity_3D consists of:
 * <ul>  		
 * 		<li> Transformation(3)  </li> 
 * 		<li> Rotation(3),  </li>
 * 		<li> Scale (3),  </li>
 * 		<li> Shear (3),  </li>
 * </ul>
 * @author Norbert Fischer
 */
public class Affinity_3D extends Homography_3D 
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. creates an identity matrix
	 */
	public Affinity_3D()  {}
	
	/**
	 * sets Affinity_3D by hand. positions are:
	 * <pre>
	 *              |aff[0] aff[3] ...    |
	 *              |aff[1] aff[4] ...    |
	 *              |aff[2] aff[5] ...    |
	 *              |  0      0     0    1|
	 * </pre>
	 * @param aff   a double array containing the values. Must be 
	 * 			    dimension 12
	 */
	public Affinity_3D( double aff[])
	{
		this (aff, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}
	
	/**
	 * sets Affinity_3D with covariance by hand. positions are:
	 * <pre>
	 *              |aff[0] aff[3] ...    |
	 *              |aff[1] aff[4] ...    |
	 *              |aff[2] aff[5] ...    |
	 *              |  0      0     0    1|
	 * </pre>
	 * @param aff   a double array containing the values. Must be 
	 * 			    dimension 12
	 * @param cov  16x16 Matrix
	 */
	public Affinity_3D( double aff[], Matrix cov)
	{
		//check dimensions
		if ( aff.length!=DIM*(DIM-1) ) 
		{
			throw new IllegalArgumentException("Array must " +
				"have 12 values!");
		}
		
		//fill the vector
		Vector tmpV = LinearAlgebraFactory.getVector(DIM*DIM);
		for(int i=0; i<DIM; i++)
		{
			for(int j=0; j<DIM-1; j++)
			{
				tmpV.set(i*DIM+j, aff[i*(DIM-1)+j]);
			}
		}
		tmpV.set((DIM*DIM)-1, 1);
		setV(tmpV);
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);		
	}
	
	
	/**
	 * Creates an affinity_3D by 12 paramaters:
	 * <ul>  		
	 * 		<li> Transformation(3)  </li> 
	 * 		<li> Rotation(3),  </li>
	 * 		<li> Scale (3), in x and y different values possible </li>
	 * 		<li> Shear (3),  </li>
	 * </ul>
	 * Error propagation is done automatically.
	 * @param rot   contains the rotation matrix
	 * @param trans   contains the translation matrix
	 * @param scale   a vector with dimension 3 
 	 * @param shear   Vector containing 6 entries with order:
 	 * 		<pre>            xy, xz, yz, yx, zx, zy </pre>
	 * 		e.g. xy means: Shear-parameter in xy-plane <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 */
	public Affinity_3D (Rotation_3D rot, Translation_3D trans,
						Vector scale, Vector shear)
	{
		this(rot, trans, scale, shear, null);
	}
	
	/**
	 * Creates an affinity_3D by 12 paramaters with covariance:
	 * <ul>  		
	 * 		<li> Transformation(3)  </li> 
	 * 		<li> Rotation(3),  </li>
	 * 		<li> Scale (3), in x and y different values possible </li>
	 * 		<li> Shear (3),  </li>
	 * </ul>
	 * No error propagation will be done.
	 * @param rot   contains the rotation matrix
	 * @param trans   contains the translation matrix
	 * @param scale   a vector with dimension 3 
 	 * @param shear   Vector containing 6 entries with order:
 	 * 		<pre>            xy, xz, yz, yx, zx, zy </pre>
 	 *      or [1  a1 a2 0]
 	 *         [a4 1  a3 0]
 	 *         [a5 a6  1 0]
 	 * 
	 * 		e.g. xy means: Shear-parameter in xy-plane <br>
	 * 		if you want symmetric shear, set xy=yx, yz=zy and xz=zx
	 * @param cov   16x16 matrix
	 */
	public Affinity_3D (Rotation_3D rot, Translation_3D trans,
						Vector scale, Vector shear,
						Matrix cov)
	{
		//check dimensions of parameter
		if ( scale.getDim()!=DIM-1 )
		{
			throw new IllegalArgumentException("scale vector has not " +
							"three entries!");
		}
		if ( shear.getDim()!=6) 
		{
			throw new IllegalArgumentException("shear vector has not " +
				"six entries!");
		}
		
		//set Rotation, Translation
		Motion_3D mot = new Motion_3D(trans, rot);
		Matrix tmpM = mot.getTransformationMatrix();

		//set scale
		tmpM.set(0,0, tmpM.get(0,0)*scale.get(0));
		tmpM.set(1,1, tmpM.get(1,1)*scale.get(1));
		tmpM.set(2,2, tmpM.get(2,2)*scale.get(2));

		//set shear
		/*
		 * Found under: 
		 * http://www.mpi-sb.mpg.de/units/ag4/teaching/uebung/lecture02.pdf
		 * http://www.cs.fit.edu/wds/classes/cse5255/thesis/shear/shear.html#3dShear
		 */
		Matrix shearMat = LinearAlgebraFactory.getMatrixId(DIM);
		shearMat.set(0,1, shear.get(0)); //xy: y amount to x 
		shearMat.set(0,2, shear.get(1)); //xz: z amount to x
		shearMat.set(1,2, shear.get(2)); //yx: x amount to y
		shearMat.set(1,0, shear.get(3)); //yz: ...
		shearMat.set(2,0, shear.get(4)); //zx:
		shearMat.set(2,1, shear.get(5)); //zy: 
		
		tmpM=tmpM.mult(shearMat);
		
		setV(tmpM.vec());
		//FFG
		/*
 		 * null comes from Homography_2D(Affinity_3D aff, Vector projective)
		 * It's a kind of flag to show, that FFG should be performed
		 */
		if (cov==null)	
		{
			//Because this homography CONSISTS of mot, scale and shear, no
			//error propagation could done (between what, if so?)			
			setCov(mot.getCov());
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else
		{
			checkCovDim(cov, DIM*DIM);
			setCov(cov);
		}
	}
	
	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 12;
	}	
		
} // end Affinity_3D



