/** Java class "Entity.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Entity is the super-class of Point_xD, Line_xD and Plane_3D. So it is a 
 * container for methods which are needed in all subclasses  
 * 
 * @author Norbert Fischer
 * @version 2.0 This version holds a methods neccessary for an
 *          estimation of elements. (T. Laebe, 25.06.2003)
 */
public abstract class Entity extends Element {

	///////////////////////////////////////
	// operations
	/**
	 * returns the euclidean part of an element
	 * @return a Vector with euclidean part in it. Recall that 
	 * 			an entity is parted in homogeneous and euclidean
	 */
	public abstract Vector getEuclideanPart();

	/**
	 * Return the (euclidean) norm of the euclidean part.
	 * @return the norm
	 */
	public double getNormOfEuclideanPart() {
		return getEuclideanPart().getLength();
	} // end getNormOfEuclideanPart

	/**
	 * returns the homogoneous part of an element
	 * @return a Vector with homogeneous part in it. Recall that 
	 * 			an entity is parted in homogeneous and euclidean
	 */
	public abstract Vector getHomogeneousPart();

	/**
	 * Return the (euclidean) norm of the homogenous part.
	 * @return the norm
	 */
	public double getNormOfHomogeneousPart() {
		// your code here
		return getHomogeneousPart().getLength();
	} // end getNormOfHomogeneousPart        

	/**
	 * returns the degrees of freedem. It is the minimum number of parameters to
	 * construct one entity <br>
	 * e.g.: If we have a Point_3D, the degree of freedom is 3
	 */
	public abstract int getDegreesOfFreedom();

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public abstract Matrix getJacobianOfRelation(
		Class relationType,
		Class partnerType);
		
	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !) This function returns
	 * the Jacobian after the first entity in the relation. Therefore
	 * *this is the second entity (The unknown in estimation, so this
	 * is the Jacobian after the observation)
	 * @param r a (Bi)-Relational Property. From this property only the 
	 *        relation type and the type of the first Entity is used.
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(RelationalProperty r) {
		BiRelationalProperty rb=(BiRelationalProperty) r;
		return getJacobianOfRelation(rb.getClass(), rb.getFirstEntity().getClass());
	}
	
	/**
	 * Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation. 
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public abstract Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row);

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param secondType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public abstract int getDofOfRelation(Class relationType, Class secondType);


	/**
	 * Normalize this element so, that homogeneous part has length 1
	 */
	public abstract void normalizeEuclidean();

	/**
	 * Euclidean normalization with comparable result: To
	 * avoid a sign ambiguity the largest element |xi| of v is
	 * positive after this normalization. This is slightly
	 * more expensive than normalizeEuclidean.
	 */
	public void normalizeEuclideanComparable() {
		normalizeEuclidean();
		double max = 0;
		for (int i = 0; i < v.getDim(); i++)
			// 1e-16 to avoid ambiguities when values are identical
			if (Math.abs(v.get(i)) + 1e-16 > Math.abs(max))
				max = v.get(i);
		if (max < 0)
			v = v.mult(-1.0);
	}

	/**
	 * Transforms this Entity with a given Transformation
	 * @param transformation   Any allowed Transformation, like Homography_xD,
	 * 						   Affinity_xD,...
	 */
	public void transform(Transformation transformation) {
		Entity e = transformation.transform(this);
		setV(e.getV());
		setCov(e.getCov());
	} // end transform        

	/**
	 * Return the distance to the origin.
	 * @return the distance to the origin.
	 */
	public double getDistanceToOrigin() {
		// your code here
		return getNormOfEuclideanPart() / getNormOfHomogeneousPart();
	} // end getDistanceToOrigin

	/**
	  * Compute the (euclidean) distance to another entity. This
	  * is of course not possible for all combinations.
	  * In the case of an illegal combination an IllegalArgumentException
	  * should be thrown.
	  * @return the distance to the given entity
	  * @param entity the entity to which a distance should be computed.
	 */
	public abstract double getDistance(Entity entity);

	/**
	 * Checks, if this element and e are the same
	 * @return true, if they are the same, and false, if not
	 * @param e the element to be tested
	 */
	public boolean isEqual(Entity e) {
		Relation r = new Relation(this, e, Equal.class);
		return r.decision();
	}

	/**
	 * checks, if this element and e are parallel
	 * @return true, if they are parallel, and false, if not
	 * @param e the element to be tested
	 */
	public boolean isParallel(Entity e) {
		Relation r = new Relation(this, e, Parallel.class);
		return r.decision();
	}

	/**
	 * checks, if this element and e are incident
	 * @return true, if they are incident, and false, if not
	 * @param e the element to be tested
	 */
	public boolean isIncident(Entity e) {
		Relation r = new Relation(this, e, Incident.class);
		return r.decision();
	}

	/**
	 * checks, if this element and e are orthogonal
	 * @return true, if they are orthogonal, and false, if not
	 * @param e the element to be tested
	 */
	public boolean isOrthogonal(Entity e) {
		Relation r = new Relation(this, e, Orthogonal.class);
		return r.decision();
	}

	/**
	 * <p>
	 * calculates the condition factor which depends on two elements.
	 * What is done: Find the element farthest away from the origin. Calculate
	 * the condition factor and apply this to all entities.
	 * Programmed after Stephan Heuels Dissertation (table 2.5 on page 37)
	 * 
	 * </p><p>
	 * 
	 * @param x  first element
	 * </p><p>
	 * @param y  second element
	 * </p><p>
	 * @return a double with containing the condition factor
	 * </p>
	 */
	public static double getConditionFactorF(Entity x, Entity y) {
		Entity[] elist = new Entity[2];
		elist[0] = x;
		elist[1] = y;
		return getConditionFactorF(elist);
	}

	/**
	 * <p>
	 * calculates the condition factor which depends on a set of elements.
	 * What is done: Find the element farthest away from the origin. Calculate
	 * the condition factor and apply this to all entities.
	 * Programmed after Stephan Heuels Dissertation (table 2.5 on page 37)
	 * with the following corrections (lae, 13.01.2004):
	 * The ratio is computed as mentioned in the text: euclidean/homogeneous.
	 * The condition is neccessary if this ration is GREATER than fmin.
	 * Then f is fmin/max_oh which corresponds to fmin*max_ho in the text.
	 * </p><p>
	 * 
	 * @param elist Array of Entities to be conditioned with the same factor
	 * </p><p>
	 * @return a double with containing the condition factor. 1.0 is
	 *         returned if no conditioning is neccessary.
	 * </p>
	 */
	public static double getConditionFactorF(Entity[] elist) {
		double f = 0.0;
		//needed. Ask Stephan why
		double fmin = 0.1;
		double epsilon_m = 1e-16; //accuracy of double

		double max_oh = -1;
		
		for (int i = 0; i < elist.length; i++) {
			//calculate the ratio between euclidean and homogeneous part
			double ratio =
				elist[i].getEuclideanPart().getLength()
					/ elist[i].getHomogeneousPart().getLength();
			if (ratio > max_oh)
				max_oh = ratio;

		}

		//is ratio big enough to scale?
		if (max_oh > fmin) {
			f = fmin / max_oh;
			if (f < epsilon_m) {
				// Determine max(euclidian-norm)
				double maxO = -1;
				for (int i = 0; i < elist.length; i++) {
					double r = elist[i].getEuclideanPart().getLength();
					if (r > maxO)
						maxO = r;
				}
				f = fmin / maxO;
			}
			return f;
		}
		
        /* Old implementation
		for (int i = 0; i < elist.length; i++) {
			//calculate the ratio between euclidean and homogeneous part
			double ratio =
				elist[i].getHomogeneousPart().getLength()
					/ elist[i].getEuclideanPart().getLength();
			if (ratio > max_ho)
				max_ho = ratio;

		}

		//is ratio big enough to scale?
		if (max_ho < fmin) {
			f = fmin * max_ho;
			if (f < epsilon_m) {
				// Determine max(euclidian-norm)
				double maxO = -1;
				for (int i = 0; i < elist.length; i++) {
					//calculate the ratio between euclidean and homogeneous part
					double r = elist[i].getEuclideanPart().getLength();
					if (r > maxO)
						maxO = r;
				}
				f = fmin / maxO;
			}
			return f;
		}
		*/
		
		return 1.0;
	}

	/** Do the conditioning of a set of entities. This is done for
	 *  avoiding bias and numerical problems for estimating and testing.
	 *  If not switched of, the conditioning in done automatically while
	 *  testing and estimating
	 * @param elist a set of Entities to condition
	 * @return the used conditioning factor
	 */
	public static double doCondition(Entity[] elist) {
		double f = Entity.getConditionFactorF(elist);

		if (f != 1) {
			for (int i = 0; i < elist.length; i++)
				elist[i].doCondition(f);
		}
		return f;
	}

	/**
	 * Condition this element with a condition factor f. The entity itself
	 * and the covariance matrix is changed. This is done to move the
	 * element closer to the euclidian origin.
	 * @param f conditioning factor
	 */
	public void doCondition(double f) {
		Matrix w=getConditionMatrix(f);
		setV(w.mult(getV()));
		setCov(w.mult(getCov()).mult(w.trans()));
	}
	

	/** Condition that element with a set of conditioning factors. For an
	 *  entity 1 conditioning factor is used. Overwritten from Element.
	 * @param f conditioning factors.
	 */
	public void doCondition(double[] f) {
		doCondition(f[0]);
	} 

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 *  This matrix multiplies the homogenous part with 1/f.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	abstract protected Matrix getConditionMatrix(double f);

	/** Change the entity (covariance not changed) in that way,
	 * that all constraints (given by getConstraintValue) are
	 * fulfilled. This is done by a Gauss-Helmert-Iterations without
	 * unknowns. 
	 */
	public void ensureConstraintsWithoutIteration() {
			Matrix G = getJacobianOfConstraint();

			v =
				v.minus(
					G.trans().mult(
						(G.mult(G.trans())).invert()).mult(
						getConstraintValue()));
	}
	
} // end Entity
