/*
 * Created on 13.03.2003
 */
package sugr;

import sugr.linalg.*;
/**
 * class containing rotation(3) constructors.
 * @author Norbert Fischer
 */
public class Rotation_3D extends Motion_3D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. Creates an empty rotation matrix = indentity 
	 * matrix 
	 */
	public Rotation_3D() {}
	
	/**
	 * Represents a rotation by three angles around the three axis'
	 * with following order of Rotationmatrices: R_z x R_y x R_x.
	 * @param omega   angle around x-Axis, given in radians
	 * @param phi   angle around y-Axis, given in radians
	 * @param kappa   angle around z-Axis, given in radians
	 */
	public Rotation_3D(double omega, double phi, double kappa)
	{
		this(omega, phi, kappa,
			 LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}

	/**
	 * Represents a rotation by three angles around the three axis'.
	 * With covariance matrix.
	 * With following order of Rotationmatrices: R_z x R_y x R_x.
	 * @param omega   angle around x-Axis, given in radians
	 * @param phi   angle around y-Axis, given in radians
	 * @param kappa   angle around z-Axis, given in radians
	 * @param cov   16x16 matrix
	 */
	public Rotation_3D(double omega, double phi, double kappa,
					   Matrix cov)
	{
		//temp values
		double cOm = Math.cos(omega); double sOm = Math.sin(omega);
		double cPhi = Math.cos(phi); double sPhi = Math.sin(phi);
		double cKap = Math.cos(kappa); double sKap = Math.sin(kappa);
		
		/*
		 * Found in:
		 * http://msl.cs.uiuc.edu/~lavalle/cs497_2001/book/geom/node10.html
		 * 
		 * Note that alpha = kappa (Z-axis), 
		 * 			 beta = phi (Y-axis), 
		 * 			 gamma = omega (X-axis)
		 */
		double [][]rotation = 
		{   {cKap*cPhi, cKap*sPhi*sOm-sKap*cOm, cKap*sPhi*cOm+sKap*sOm, 0},
			{sKap*cPhi, sKap*sPhi*sOm+cKap*cOm, sKap*sPhi*cOm-cKap*sOm, 0},
			{ -sPhi,          cPhi*sOm,                cPhi*cOm,        0},
			{   0,				0,						 0,			    1}
		};
		
		setV(LinearAlgebraFactory.getMatrix(rotation).vec());
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}

	
	/**
	 * representation of rotation through RODRIGUEZ
	 * Please note: rotations with 180 are not allowed
	 * 
	 * @param   spinAxis - A 3x1-Vector containing the axis after RODRIGUEZ
	 * 			Read more about this in Frstner, PhotoI, page 5-7 
	 */
	public Rotation_3D(Vector spinAxis)
	{
		this(spinAxis, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}
	
	/**
	 * representation of rotation through RODRIGUEZ with covariance 
	 * (only covariance of rotation matrix)
	 * Please note: rotations with 180 are not allowed
	 * 
	 * Found in:
	 * http://www.ipb.uni-bonn.de/publications/papers99/foerstner99_rotation.ps.gz
	 * @param spinAxis   A 3x1-Vector containing the axis after RODRIGUEZ.
	 * @param cov   16x16 Matrix 
	 */
	public Rotation_3D(Vector spinAxis, Matrix cov)
	{
		/*
		 * if i undastood this, each parametrisation of rotation is linked with
		 * each other (see source). this does mean (table 1):
		 * 		Rodriguez given -> quaternion
		 * 		(a,b,c)		    -> (1, a/2, b/2, c/2)
		 */
		this(1, 
			 spinAxis.get(0)/2.0, 
			 spinAxis.get(1)/2.0, 
			 spinAxis.get(2)/2.0, 
			 cov);
		//VERIFY THIS
	}
	
	/**
	 * representation of rotation with quaternions. This is the best case
	 * if you want to use all angles (no special cases)
	 * @param q0   first parameter of quaternion
	 * @param q1   second parameter of quaternion
	 * @param q2   third parameter of quaternion
	 * @param q3   fourth parameter of quaternion
	 */
	public Rotation_3D(double q0, double q1, double q2, double q3)
	{
		this(q0, q1, q2, q3,
			 LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}

	/**
	 * representation of rotation with quaternions and covariance (of 
	 * rotation matrix!) . This is 
	 * the best representation if you want to use all angles (no special cases)
	 * TODO: Use covariance of quaternion and do error propagation !
	 * @param q0   first parameter of quaternion
	 * @param q1   second parameter of quaternion
	 * @param q2   third parameter of quaternion
	 * @param q3   fourth parameter of quaternion
	 * @param cov   16x16 Matrix
	 */
	public Rotation_3D(double q0, double q1, double q2, double q3,
					   Matrix cov)
	{
		//norm quaternion
		double norm = Math.sqrt(q0*q0+q1*q1+q2*q2+q3*q3);
		q0 /= norm;
		q1 /= norm;
		q2 /= norm;
		q3 /= norm;
		
		/*
		 * Found in:
		 * http://www.ipb.uni-bonn.de/publications/papers99/foerstner99_rotation.ps.gz
		 */
		double [][]rotation = 
		{   {q0*q0+q1*q1-q2*q2-q3*q3,      2*(q1*q2-q0*q3), 	2*(q1*q3+q0*q2), 		 0},
			{   2*(q2*q1+q0*q3), 	  q0*q0-q1*q1+q2*q2-q3*q3,  2*(q2*q3-q0*q1), 		 0}, 
			{   2*(q3*q1-q0*q2),  		   2*(q3*q2+q0*q1), 	q0*q0-q1*q1-q2*q2+q3*q3, 0},
			{     0,							0,						0,			     1}
		};
		
		setV(LinearAlgebraFactory.getMatrix(rotation).vec());
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}
	
	/**
	 * Rotate around any abitrary axix with angle
	 * @param axis   a vector with three elements (x,y,z)
	 * @param angle   given in radians
	 */
	public Rotation_3D(Vector axis, double angle)
	{
		this(axis, angle, LinearAlgebraFactory.getMatrix(DIM*DIM, DIM*DIM));
	}
	
	/**
	 * Rotate around any abitrary axix with angle, with covariance
	 * @param axis   a vector with three elements (x,y,z)
	 * @param angle   given in radians
	 * @param cov   16x16 Matrix
	 */
	public Rotation_3D(Vector axis, double angle, Matrix cov)
	{
		if (axis.getDim()!=DIM-1) 
		{
			throw new IllegalArgumentException("axis vector must " +
				"have three entries!");
		}
		
		//normalize axis
		axis = axis.mult(1.0/axis.getLength());
		double x= axis.get(0);
		double y= axis.get(1);
		double z= axis.get(2);
		
		//temp values, easier to write
		double c = Math.cos(angle);
		double s = Math.sin(angle);
		double t = 1-c;
		
		/*
		 * source found in:
		 * http://www.makegames.com/3drotation/
		 */
		double [][] rotation = { { t*x*x+c,   t*x*y-s*z, t*x*z+s*y, 0},
								 { t*x*y+s*z, t*y*y+c,   t*y*z-s*x, 0},
								 { t*x*z-s*y, t*y*z+s*x, t*z*z+c, 	0},
								 {     0,		  0,		0,		1}
							   };
								
		setV(LinearAlgebraFactory.getMatrix(rotation).vec());								
		
		checkCovDim(cov, DIM*DIM);
		setCov(cov);
	}
	
	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 3;
	}	
}
