/*
 * Created on 17.11.2004
 */
package sugr;

import sugr.linalg.*;
/**
 * This matrix contains Translation(3) and Rotation(3) and one Scalefactor
 * for all three axis in 3D
 * 
 * @author Achim Knigs
 */
public class ScaledMotion_3D extends Affinity_3D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. Creates an identity matrix
	 *
	 */
	public ScaledMotion_3D()	{}
	
	/**
	 * creates a motion by a given Translation_3D and a Rotation_3D
	 * and a scale factor.
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 * @param scale contains scale factor
	 */
	public ScaledMotion_3D (Translation_3D trans, Rotation_3D rot, double scale)
	{
		this(trans, rot,scale, null);
	}

	/**
	 * creates a motion with covariance by a given Translation_3D 
	 * and a Rotation_3D and a scale factor.
	 * @param trans   contains the Translation-matrix
	 * @param rot   contains the rotation-matrix
	 * @param scale contains scale factor
	 * @param cov   16x16 matrix
	 */
	public ScaledMotion_3D (Translation_3D trans, Rotation_3D rot,double scale,
					  Matrix cov)
	{
		Matrix sc = LinearAlgebraFactory.getDiagMatrix(4,scale);
		sc.set(3,3,1);
		setV(rot.getTransformationMatrix().
		  mult(trans.getTransformationMatrix().mult(sc)).vec());
		
		//FFG
		if (cov==null)	
		{
			setCov(errorProp(rot, trans));
		}
		//if not null, then cov must be given by user und therefore be checked ;-)
		else
		{
			checkCovDim(cov, DIM*DIM);
			setCov(cov);
		}
	}

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 7;
	}	
	
	/**
	 * Compute an approximation of an entity. The result of this
	 * method approximates the optimal solution which fullfills all
	 * the given relational constraints in an optimal way. The old values
	 * of the entity are overwritten.
	 *  
	 * This is an overwritten method from Entity. In this implementation
	 * the projective parts of the transformation matrix are left out,
	 * because they are fixed to 0 (we have an affinity transformation!)
	 * and need NOT be estimated. So there
	 * are not 15 but only 12 parameters estimated which delivers a
	 * better estimate.
	 * Be careful if the number of observations is smaller than 12 ! Then
	 * the approximation is likely to fail. This is the case with e.g. 
	 * 3 points which deliver only 3*3=9 constraints (which is sufficient for
	 * a scaled motion, but not of this approximation which delivers an
	 * affinity tranformation!) 
	 * 
	 * This approxmation can be put in Affinity_3D, because it delivers an
	 * affinity. But currently no constraints are implemented in Affinity_3D,
	 * so that an estimation of Affinity_3D delivers a homography_3D. As
	 * long as no constraints are implemented, leave this approximation here,
	 * because then all estimation related methods are together.
	 * 
	 * No conditioning is done !
	 * 
	 * @param rprops a set of relational Properties which should be fullfilled.
	 */
	public void approximate(RelationalProperty[] rprops) {
		Matrix Ai = leaveOutProjectivePartsOfJacobian(rprops[0].getJacobian());
		
		Matrix A  = Ai.trans().mult(Ai);
			
		for (int i = 1; i < rprops.length; i++) {
			Ai = leaveOutProjectivePartsOfJacobian(rprops[i].getJacobian());
			A = A.plus(Ai.trans().mult(Ai));
		}
		Matrix[] svd = A.svd();

		Vector x = svd[2].getColumn(svd[2].getColumnDimension() - 1);
		
		double [] xd={x.get(0), x.get(1), x.get(2), 0, 
				x.get(3), x.get(4), x.get(5), 0,
				x.get(6), x.get(7), x.get(8), 0,
				x.get(9), x.get(10), x.get(11), x.get(12)};
		v = LinearAlgebraFactory.getVector(xd);
		
		cov = LinearAlgebraFactory.getMatrix(v.getDim(), v.getDim());
	}
	
	/** Leave out those columns of the Jacobian of a ScaledMotion estimation
	 * which correspond to the 3 projective values (the last row of the
	 * transformation matrix)
	 * @param A a nx16 Jacobian matrix
	 * @return A without projective parts (nx13)
	 */
	private static Matrix leaveOutProjectivePartsOfJacobian(Matrix A) {
		int [] usedc={0,1,2,4,5,6,8,9,10,12,13,14,15};
		int [] usedr=new int[A.getRowDimension()];
		for(int i=0; i<A.getRowDimension();i++)
			usedr[i]=i;
		
		return A.slice(usedr,usedc);
	}


	/** Get the value of the constraint (h(beta) or g(y))  if this entity 
	 * is an unknown or an observation.
	 * This is, that the last row is 0 0 0 x ,
	 * all rows are orthogonal to each other,
	 * the first three rows have the same length,
	 * and the whole vector has length 1
	 * @return the value of the constraint as a vector.
	 */
	protected Vector getConstraintValue() {
		Vector vhelp = LinearAlgebraFactory.getVector(9);
		Matrix trans = this.getTransformationMatrix();
		
		//length constraint for the whole vector
		vhelp.set(0,0.5*(v.scalarmult(v) - 1));

		//zero entries
		vhelp.set(1, trans.get(3,0));
		vhelp.set(2, trans.get(3,1));
		vhelp.set(3, trans.get(3,2));
		
		//orthogonal constraint
		trans.set(3,0,0);
		trans.set(3,1,0);
		trans.set(3,2,0);
		vhelp.set(4,trans.getColumn(0)
				.scalarmult(trans.getColumn(1)));
		vhelp.set(5,trans.getColumn(1)
				.scalarmult(trans.getColumn(2)));
		vhelp.set(6,trans.getColumn(0)
				.scalarmult(trans.getColumn(2)));
		
		//length constraint
		double s = trans.get(3,3);
		s *= s;
		double row1 = trans.getColumn(0)
				.scalarmult(trans.getColumn(0));
		double row2 = trans.getColumn(1)
				.scalarmult(trans.getColumn(1));
		double row3 = trans.getColumn(2)
				.scalarmult(trans.getColumn(2));
		vhelp.set(7,row1-row2);
		vhelp.set(8,row1-row3);
		
		return vhelp;
	}
	
	/** Get the Jacobian of the constraint (H, G) if this entity 
	 *  is an unkown or an observation. 
	 * 
	 * @return the Jacobian as a matrix.
	 */
	public Matrix getJacobianOfConstraint() {
		//get Matrix filled with zeros
		Matrix jac = LinearAlgebraFactory.getMatrix(9,16);
		
		//length constraint for whole vector
		jac.setRow(0,v);
		
		//zero entries, (identity)' = 1
		jac.set(1,3,1);
		jac.set(2,7,1);
		jac.set(3,11,1);
		
		//orthogonal constraint,
		//(0 * 4 + 1 * 5 + 2 * 6)
		jac.set(4,0,v.get(4));
		jac.set(4,1,v.get(5));
		jac.set(4,2,v.get(6));
		jac.set(4,4,v.get(0));
		jac.set(4,5,v.get(1));
		jac.set(4,6,v.get(2));
		//(4 * 8 + 5 * 9 + 6 * 10)
		jac.set(5,4,v.get(8));
		jac.set(5,5,v.get(9));
		jac.set(5,6,v.get(10));
		jac.set(5,8,v.get(4));
		jac.set(5,9,v.get(5));
		jac.set(5,10,v.get(6));
		//(0 * 8 + 1 * 9 + 2 * 10)
		jac.set(6,0,v.get(8));
		jac.set(6,1,v.get(9));
		jac.set(6,2,v.get(10));
		jac.set(6,8,v.get(0));
		jac.set(6,9,v.get(1));
		jac.set(6,10,v.get(2));
		
		//length constraint for three columns
		//first column == second column
		jac.set(7,0,2*v.get(0));
		jac.set(7,1,2*v.get(1));
		jac.set(7,2,2*v.get(2));
		jac.set(7,4,-2*v.get(4));
		jac.set(7,5,-2*v.get(5));
		jac.set(7,6,-2*v.get(6));
		//first column == third column
		jac.set(8,0,2*v.get(0));
		jac.set(8,1,2*v.get(1));
		jac.set(8,2,2*v.get(2));
		jac.set(8,8,-2*v.get(8));
		jac.set(8,9,-2*v.get(9));
		jac.set(8,10,-2*v.get(10));
		
		return jac;
	}
}
