
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Tests for class Plane_3D.
 * Most important are the constructors which construct 
 * a plane out of other entities.
 * The rest is quite similar to tests for Point_3D.
 * Uses: TestStatistics.class and TestTools.class.
 * 
 * @author Achim Knigs
 */
public class Plane_3DTest extends TestCase {

	/**
	 * Constructor for Plane_3DTest.
	 * @param arg0
	 */
	public Plane_3DTest(String arg0) {
		super(arg0);
	}

	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 100 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 */
	public static int MAXITER = 100000;
	
	/**
	 * Method to test the statistic reasoning of a plane out of 
	 * an uncertain line and an uncertain point.
	 * @param point - the point
	 * @param line - the line to be connected to the point
	 * @return - true if test is successful
	 */
	public boolean helpPlane_3DPoint_3DLine_3DStatistics(Point_3D point, Line_3D line,double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(point.getV(),point.getCov());
		RandomVector vl2 = new RandomVector(line.getV(),line.getCov());
		
		//a temp point
		Plane_3D cross;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Testing reasoning of a Plane","out of a Point and a Line",0,MAXITER);
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			//System.out.println(i);
			//get "gaussian" lines
			Point_3D tpoint = new Point_3D(vl1.getNextRandom(),point.getCov());
			Line_3D tline = new Line_3D(vl2.getNextRandom(),line.getCov());
			
			//calculate point and add observation
			cross = new Plane_3D(tpoint,tline);
			stat.addObservation(cross);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//calculate values
		cross = new Plane_3D(point,line);
		
		Plane_3D estimated = new Plane_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross,estimated,sigma);
		
		if(!retval) {
			System.out.println("Constructor Failed: Plane_3D(Point_3D,Line_3D)");
			System.out.println("Failed Plane: " + cross);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}
	
	/**
	 * Method to test the statistic reasoning of a plane out of 
	 * an uncertain line and an uncertain point.
	 * @param line - the line to be connected to the point
	 * @param point - the point
	 * @return - true if test is successful
	 */
	public boolean helpPlane_3DLine_3DPoint_3DStatistics(Line_3D line, Point_3D point, double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(point.getV(),point.getCov());
		RandomVector vl2 = new RandomVector(line.getV(),line.getCov());
		
		//a temp point
		Plane_3D cross;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Testing reasoning of a Plane","out of a Line and a Point",0,MAXITER);
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			//System.out.println(i);
			//get "gaussian" lines
			Point_3D tpoint = new Point_3D(vl1.getNextRandom(),point.getCov());
			Line_3D tline = new Line_3D(vl2.getNextRandom(),line.getCov());
			
			//calculate point and add observation
			cross = new Plane_3D(tline,tpoint);
			stat.addObservation(cross);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//calculate values
		cross = new Plane_3D(line,point);
		
		Plane_3D estimated = new Plane_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross,estimated,sigma);
		
		if(!retval) {
			System.out.println("Failed Constructor: Plane_3D(Line_3D,Point_3D)");
			System.out.println("Failed Plane: " + cross);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}
	
	/**
	 * Method to test the statistic reasoning of a plane out of 
	 * two uncertain lines.
	 * @param line1 - first line to be crossed with second line
	 * @param line2 - second line
	 * @return - true if test is successful
	 */
	public boolean helpPlane_3DLine_3DLine_3DStatistics(Line_3D line1, Line_3D line2,double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(line1.getV(),line1.getCov());
		RandomVector vl2 = new RandomVector(line2.getV(),line2.getCov());
		
		//a temporary plane
		Plane_3D cross;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
						"Testing reasoning of a Plane","out of two Lines",0,MAXITER/100);
		
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER/100; i++)
		{
			//System.out.println(i);
			//get "gaussian" lines
			Line_3D tline1 = new Line_3D(vl1.getNextRandom(),line1.getCov());
			Line_3D tline2 = new Line_3D(vl2.getNextRandom(),line2.getCov());
			
			//calculate point and add observation
			cross = new Plane_3D(tline1,tline2);
			stat.addObservation(cross);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);			
		}
		pm.close();
		
		//calculate values
		cross = new Plane_3D(line1,line2);
		
		Plane_3D estimated = new Plane_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(cross,estimated,sigma);
		
		if(!retval) {
			System.out.println("Constructor Failed: Plane_3D(Line_3D,Line_3D)");
			System.out.println("Failed Plane: " + cross);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}
	
	/**
	 * Method to test the statistic reasoning of a plane out of
	 *  three uncertain points.
	 * @param point1 - the first point
	 * @param point2 - the second point
	 * @param point3 - the third point
	 * @return - true if test is successful
	 */
	public boolean helpPlane_3DPoint_3DPoint_3DPoint_3DStatistics(Point_3D point1, Point_3D point2, Point_3D point3, double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(4);
		
		//Two Random Vectors for Random Point generation
		RandomVector vl1 = new RandomVector(point1.getV(),point1.getCov());
		RandomVector vl2 = new RandomVector(point2.getV(),point2.getCov());
		RandomVector vl3 = new RandomVector(point3.getV(),point3.getCov());
		
		//a temp point
		Plane_3D cross;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
						"Testing reasoning of a Plane","out of three Points",0,MAXITER);
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			//get "gaussian" points
			Point_3D tpoint1 = new Point_3D(vl1.getNextRandom(),point1.getCov());
			Point_3D tpoint2 = new Point_3D(vl2.getNextRandom(),point2.getCov());
			Point_3D tpoint3 = new Point_3D(vl3.getNextRandom(),point3.getCov());
			
			//calculate point and add observation
			cross = new Plane_3D(tpoint1,tpoint2,tpoint3);
			stat.addObservation(cross);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);	
		}
		pm.close();
		
		//calculate values
		cross = new Plane_3D(point1,point2,point3);
		
		Plane_3D estimated = new Plane_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		cross.normalizeEuclidean();
		estimated.normalizeEuclidean();


		//compare and return true if equal
		boolean retval = TestStatistics.check(cross,estimated,sigma);
		
		if(!retval) {
			System.out.println("Failed Constructor: Plane_3D(Point_3D,Point_3D,Point_3D)");
			System.out.println("Failed Plane: " + cross);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}
	
	/**
	 * Test for getDegreesOfFreedom.
	 */
	public void testGetDegreesOfFreedom() {
		//construct planes
		Plane_3D p1 = new Plane_3D(2,3,4,1);
		Plane_3D p2 = new Plane_3D(1.5,7.4,6.4);
		
		//use getDegreesOfFreedom()
		double dof1 = p1.getDegreesOfFreedom();
		double dof2 = p2.getDegreesOfFreedom();
		
		//test values
		if(dof1 != 3) fail();
		if(dof1 != dof2) fail();		
	}

	/**
	 * Test for getDimension.
	 */
	public void testGetDimension() {
		//construct planes
		Plane_3D p1 = new Plane_3D(5,4.3251,3,4,LinearAlgebraFactory.getDiagMatrix(4,0.05));
		Plane_3D p2 = new Plane_3D(7,5,465);
		
		//use getDimension()
		int dim1 = p1.getDimension();
		int dim2 = p2.getDimension();
		
		//test values
		if(dim1 != 4) fail();
		if(dim1 != dim2) fail();
	}

	/**
	 * Test for getEuclideanPart
	 */
	public void testGetEuclideanPart() {
		//construct planes
		Plane_3D p1 = new Plane_3D(1,Double.MIN_VALUE,3,Double.MAX_VALUE);
		Plane_3D p2 = new Plane_3D(-43,21.6,25*3,Double.MIN_VALUE);
		
		//use getEuclideanPart()
		Vector eucl1 = p1.getEuclideanPart();
		Vector eucl2 = p2.getEuclideanPart();
		
		//fail if not null
		if(eucl1.get(0) != Double.MAX_VALUE) fail();
		if(eucl2.get(0) != Double.MIN_VALUE) fail();
	}

	/**
	 * Test for getHomogeneousPart()
	 */
	public void testGetHomogeneousPart() {
		//construct planes
		Plane_3D p1 = new Plane_3D(1,2,3,Double.MAX_VALUE);
		Plane_3D p2 = new Plane_3D(-43,21.6,25*3,0);
		Plane_3D p3 = new Plane_3D(Double.MIN_VALUE,5,2);
		
		//use getEuclideanPart()
		Vector hom1 = p1.getHomogeneousPart();
		Vector hom2 = p2.getHomogeneousPart();
		Vector hom3 = p3.getHomogeneousPart();
		
		//preparing test
		Vector kontr1 = LinearAlgebraFactory.getVector(1,2,3);
		Vector kontr2 = LinearAlgebraFactory.getVector(-43,21.6,75);
		Vector kontr3 = LinearAlgebraFactory.getVector(Double.MIN_VALUE,5,2);
		Vector diff1 = hom1.minus(kontr1);
		Vector diff2 = hom2.minus(kontr2);
		Vector diff3 = hom3.minus(kontr3);
		
		//control values
		if(diff1.getLength() >= 0.00000001) fail();
		if(diff2.getLength() >= 0.00000001) fail();
		if(diff3.getLength() >= 0.00000001) fail();
	}

	/**
	 * Test for Matrix getJacobianOfRelation(Class, Class)
	 * First tests if the jacobians are right with some planes,
	 * lines and points using the constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0
	 * then tests all jacobian matrices value for value.
	 */
	public void testGetJacobianOfRelationClassClass() {
		//constructing planes

		//two planes which are equal
		Plane_3D eqPlane1 = new Plane_3D(2,-4,6,0);
		Plane_3D eqPlane2 = new Plane_3D(1,-2,3,0);
		//a plane that is parallel to the equal planes
		Plane_3D parallelPlane = new Plane_3D(1,-2,3,1);			 		
		//a line that is parallel to the equal planes
		Line_3D parallelLine = new Line_3D(-2,-1,0,0,-1,2);
		//a plane that is orthogonal to the equal planes
		Plane_3D orthoPlane = new Plane_3D(1,2,1,0);
		//a line that is orthogonal to the equal planes
		Line_3D orthoLine = new Line_3D(1,-2,3,-1,0,0);
	
		//a plane.
		Plane_3D contentPlane = new Plane_3D(1,0,0,1);
		//a point on that plane
		Point_3D pointPlane = new Point_3D(-1,12,9,1);
		//a line on that plane
		Line_3D linePlane = new Line_3D(0,-7,11,21,11,7);

		//getting the Jacobians
		Matrix planeEqMatrix = eqPlane1.getJacobianOfRelation(Equal.class, Plane_3D.class);
		Matrix planeParallelMatrix = eqPlane1.getJacobianOfRelation(Parallel.class, Plane_3D.class);
		Matrix lineParallelMatrix = eqPlane1.getJacobianOfRelation(Parallel.class, Line_3D.class);
		Matrix planeOrthoMatrix = eqPlane1.getJacobianOfRelation(Orthogonal.class, Plane_3D.class);
		Matrix lineOrthoMatrix = eqPlane1.getJacobianOfRelation(Orthogonal.class, Line_3D.class);
		Matrix pointOnPlaneMatrix = contentPlane.getJacobianOfRelation(Incident.class, Point_3D.class);
		Matrix lineOnPlaneMatrix = contentPlane.getJacobianOfRelation(Incident.class, Line_3D.class);
		
		//test the Jacobians, that means, test if they fullfill the constraint:
		Matrix planeEq = planeEqMatrix.mult(eqPlane2.getV().getMatrix());
		Matrix planeParallel = planeParallelMatrix.mult(parallelPlane.getV().getMatrix());
		Matrix lineParallel = lineParallelMatrix.mult(parallelLine.getV().getMatrix());
		Matrix planeOrtho = planeOrthoMatrix.mult(orthoPlane.getV().getMatrix());
		Matrix lineOrtho = lineOrthoMatrix.mult(orthoLine.getV().getMatrix());
		Matrix incPoint = pointOnPlaneMatrix.mult(pointPlane.getV().getMatrix());
		Matrix incLine = lineOnPlaneMatrix.mult(linePlane.getV().getMatrix());
		
		//fail if not null:
		if(planeEq.vec().getLength() != 0) fail();
		if(planeParallel.vec().getLength() != 0) fail();
		if(lineParallel.vec().getLength() != 0) fail();
		if(planeOrtho.vec().getLength() != 0) fail();
		if(lineOrtho.vec().getLength() != 0) fail();
		if(incPoint.vec().getLength() != 0) fail();
		if(incLine.vec().getLength() != 0) fail();				
		
		
		//try every forbidden combination:
		Class[][] tests = {	{Equal.class,Plane_3D.class},
							{Incident.class,Line_3D.class},
							{Incident.class,Point_3D.class},
							{Parallel.class,Plane_3D.class},
							{Parallel.class,Line_3D.class},
							{Orthogonal.class,Plane_3D.class},
							{Orthogonal.class,Line_3D.class}}; 
		
		if(!TestTools.helpJacobianOfRelations(tests,eqPlane1)) fail();
		
		//calculate Jacobians and compare them to the ones form sugr
		//a new Point
		Plane_3D A = new Plane_3D(6,9,3);
		
		//let sugr calculate the matrices
		Matrix incPoint1 = A.getJacobianOfRelation(Incident.class, Point_3D.class);
		Matrix incLine1 = A.getJacobianOfRelation(Incident.class, Line_3D.class);
		Matrix eqP1 = A.getJacobianOfRelation(Equal.class, Plane_3D.class);
		Matrix parallelPlane1 = A.getJacobianOfRelation(Parallel.class, Plane_3D.class);
		Matrix parallelLine1 = A.getJacobianOfRelation(Parallel.class, Line_3D.class);
		Matrix orthoPlane1 = A.getJacobianOfRelation(Orthogonal.class, Plane_3D.class);
		Matrix orthoLine1 = A.getJacobianOfRelation(Orthogonal.class, Line_3D.class);
		
		//do it yourself:
		//incident plane and point
		Matrix incPoint2 = A.getV().getMatrix().trans();
		
		//incident plane and line
		//Pi(A)^T 
		double [][] values = {	{1,0,0,-6},
								{0,1,0,-9},
								{0,0,1,-3},
								{0,-3,9,0},
								{3,0,-6,0},
								{-9,6,0,0}};
		Matrix incLine2 = LinearAlgebraFactory.getMatrix(values).trans();
		
		//plane equals plane
		//PiDual(A)
		double [][] values2 = {	{0,3,-9,0},
								{-3,0,6,0},
								{9,-6,0,0},
								{-1,0,0,6},
								{0,-1,0,9},
								{0,0,-1,3}};
		Matrix eqP2 = LinearAlgebraFactory.getMatrix(values2);
		
		//plane parallel to plane
		//S(A_h)
		double[][] values3 = {	{0,-3,9,0},
								{3,0,-6,0},
								{-9,6,0,0},
								{0,0,0,0}};
		Matrix parallelPlane2 = LinearAlgebraFactory.getMatrix(values3);
		
		//line parallel to plane
		//A_h^T
		double[][] values4 = {	{6},
								{9},
								{3},
								{0},
								{0},
								{0}};
		Matrix parallelLine2 = LinearAlgebraFactory.getMatrix(values4).trans();
		
		//plane orthogonal to plane
		//A_h^T
		double[][] values5 = {	{6},
								{9},
								{3},
								{0}};
		Matrix orthoPlane2 = LinearAlgebraFactory.getMatrix(values5).trans();
		
		//line orthogonal to plane
		//-S(A_h)
		double[][] values6 = {	{0,-3,9,0},
								{3,0,-6,0},
								{-9,6,0,0},
								{0,0,0,0},
								{0,0,0,0},
								{0,0,0,0}};
		Matrix orthoLine2 = LinearAlgebraFactory.getMatrix(values6).trans();
		
		//test if equal:
		if((incPoint1.minus(incPoint2).vec()).getLength() != 0) fail();
		if((incLine1.minus(incLine2).vec()).getLength() != 0) fail();
		if((eqP1.minus(eqP2).vec()).getLength() != 0) fail();
		if((parallelPlane1.minus(parallelPlane2).vec()).getLength() != 0) fail();
		if((parallelLine1.minus(parallelLine2).vec()).getLength() != 0) fail();
		if((orthoPlane1.minus(orthoPlane2).vec()).getLength() != 0) fail();
		if((orthoLine1.minus(orthoLine2).vec()).getLength() != 0) fail();								
	}

	/**
	 * Test for getDofOfRelation
	 * Only one plane, because value doesn't base on values of that plane
	 */
	public void testGetDofOfRelation() {
		//construct plane
		Plane_3D plane = new Plane_3D(1,2,3);
		
		//getDofOfRelations
		int dofEq = plane.getDofOfRelation(Equal.class, Plane_3D.class);
		int dofIncPoint = plane.getDofOfRelation(Incident.class, Point_3D.class);
		int dofIncLine = plane.getDofOfRelation(Incident.class, Line_3D.class);
		int dofPPlane = plane.getDofOfRelation(Parallel.class, Plane_3D.class);
		int dofPLine = plane.getDofOfRelation(Parallel.class, Line_3D.class);
		int dofOrthoPlane = plane.getDofOfRelation(Orthogonal.class, Plane_3D.class);
		int dofOrthoLine = plane.getDofOfRelation(Orthogonal.class, Line_3D.class);
		
		//test values
		if(dofEq != 3) fail();
		if(dofIncPoint != 1) fail();
		if(dofIncLine  != 2) fail();
		if(dofPPlane != 2) fail();
		if(dofPLine != 1) fail();
		if(dofOrthoPlane != 1) fail();
		if(dofOrthoLine != 2) fail();
	}

	/**
	 * Test of normalizeEuclidean
	 * First test the calculation of sure values over one million 
	 * different points.
	 * Then test the calculation of covariances with help of 
	 * iteration for some points.
	 */
	public void testNormalizeEuclidean() {
		//progress monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
								"Calculation of euclidean normalizations.","No statistics now",0,1000000);
		
		//Test over 1 Million Points
		outer:
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				Plane_3D plane = new Plane_3D(i-100,(j+100)%1000,i-j,j-500.5);
				double factor = 1.0/(plane.getHomogeneousPart()).getLength();
				plane.normalizeEuclidean();
				Vector v = LinearAlgebraFactory.getVector((i-100)*factor,((j+100)%1000)*factor,(i-j)*factor,(j-500.5)*factor);
				if(!TestStatistics.check(plane.getV(),v,0.00001)) {
					System.out.println("Sugr Dist: " + plane.getV());
					System.out.println("My Dist: " + v);
					pm.close();
					fail();
				}
				if(pm.isCanceled()) break outer;
				else pm.setProgress(i*j);
			} 
		}
		pm.close();
		
		//Test Statistics
		double[][] cov = {{0.2,0.01,0.01,0.01},{0.01,0.3,0.01,0.01},{0.01,0.01,0.4,0.01},{0.01,0.01,0.01,0.10502}};
		Plane_3D p1 = new Plane_3D(13.6,-22.12,73.5,654.6,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p1,MAXITER,0.25)) fail();
		
		Plane_3D p2 = new Plane_3D(264,654,523,-6754,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p2,MAXITER)) fail();
		
		Plane_3D p3 = new Plane_3D(-54,76,28,284,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p3,MAXITER)) fail();
	}

	/**
	 * Test for getDistance
	 * Tests the Distance between a point and a plane one million times.
	 */
	public void testGetDistance() {
		//progress monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
						"Calculating Distance","between Points and Planes",0,1000000);
		
		//test distance one million times:
		outer:
		for(int i = 0; i < 100; i++) {
			for(double j = 0; j < 50; j += 0.5){
				for(int k = 100; k > 0; k--) {
					//construct planes
					Plane_3D plane = new Plane_3D(i,j,k,i);
					Point_3D point = new Point_3D(j,k,i,j);
					
					//distance to first plane:
					Vector z = plane.getV().getMatrix().trans().mult(point.getV());
					Vector n = plane.getHomogeneousPart().mult(point.getHomogeneousPart().get(0));
					double myDist = z.normF() / n.normF();
					double sugrDist = plane.getDistance(point); 
					
					//test
					if(Math.abs(myDist-sugrDist) > 0.00000001) {
						pm.close();
						fail();
					} 
					
					if(pm.isCanceled()) break outer;
					else pm.setProgress(i*(int)(j*2)*k);
				}
			}
		}
		pm.close();
	}

	/**
	 * Test for void Plane_3D(Vector, Matrix)
	 */
	public void testPlane_3DVectorMatrix() {
		//get some matrix and vector with some values
		double [][] values = {	{0.1,0.002,0.002,0.002},
								{0.002,0.100562,0.002,0.002},
								{0.002,0.002,0.09954,0.002},
								{0.002,0.002,0.002,0.13}};
		Matrix cov = LinearAlgebraFactory.getMatrix(values);
		Vector v = LinearAlgebraFactory.getVector(7,Double.MAX_VALUE, Double.MIN_VALUE, 1);
		
		//contruct a plane
		Plane_3D plane = new Plane_3D(v,cov);
		
		//test if values are equal... 
		if(!TestStatistics.check(v,plane.getV(),cov,
				plane.getCov(),0.00000001)) fail();
	}

	/**
	 * Test for void Plane_3D(double, double, double, double)
	 */
	public void testPlane_3Ddoubledoubledoubledouble() {
		//generate plane
		Plane_3D plane = new Plane_3D(6,Double.MAX_VALUE,1.7,Double.MIN_VALUE);
		
		//help vector
		Vector v = plane.getV();
		Vector kontr = LinearAlgebraFactory.getVector(6, Double.MAX_VALUE,1.7,Double.MIN_VALUE);
		 
		//test if all values are equal
		if(!TestStatistics.check(kontr, v, 0.000001)) fail();
	}

	/**
	 * Test for void Plane_3D(double, double, double, double, Matrix)
	 */
	public void testPlane_3DdoubledoubledoubledoubleMatrix() {
		//get cov values and Matrix
		double [][] values = {	{0.1,0.002,0.002,0.002},
								{0.002,0.100562,0.002,0.002},
								{0.002,0.002,0.09954,0.002},
								{0.002,0.002,0.002,0.13}};
		Matrix cov = LinearAlgebraFactory.getMatrix(values);
		
		//construct plane
		Plane_3D p = new Plane_3D(1.7,5.1,-52.12,3,cov);
		
		//help vector
		Vector v = LinearAlgebraFactory.getVector(1.7,5.1,-52.12,3);
		
		//test if all values are equal:
		if(!TestStatistics.check(v, p.getV(), cov, p.getCov(),0.0000001)) fail(); 
	}

	/**
	 * Test for void Plane_3D(double, double, double)
	 */
	public void testPlane_3Ddoubledoubledouble() {
		//construct plane 
		Plane_3D p = new Plane_3D(4.5,12,6.7);
		
		//help vector
		Vector v = LinearAlgebraFactory.getVector(4.5, 12,6.7,1);
		
		//test if all values are equal:
		if(!TestStatistics.check(v, p.getV(),0.00001)) fail(); 		
	}

	/**
	 * Test for void Plane_3D(double, double, double, Matrix)
	 */
	public void testPlane_3DdoubledoubledoubleMatrix() {
		//get cov values and Matrix
		double [][] values = {	{0.1,0.002,0.002,0.002},
								{0.002,0.100562,0.002,0.002},
								{0.002,0.002,0.09954,0.002},
								{0.002,0.002,0.002,0.13}};
		Matrix cov = LinearAlgebraFactory.getMatrix(values);
		
		//construct point
		Plane_3D p = new Plane_3D(Double.MAX_VALUE,52,Double.MIN_VALUE,cov);
		
		//help vector
		Vector v = LinearAlgebraFactory.getVector(Double.MAX_VALUE,52,Double.MIN_VALUE,1);
		
		//test if all values are equal:
		if(!TestStatistics.check(v, p.getV(), cov, p.getCov(),0.0000001)) fail(); 	
	}

	/**
	 * Test for void Plane_3D(Point_3D, Line_3D)
	 * Tests the reasoning of a plane out of a point and a line 
	 * with help of TestStatistics.
	 */
	public void testPlane_3DPoint_3DLine_3D() {		
		//coariances for point
		Matrix covPMatrix = 
			LinearAlgebraFactory.getDiagMatrix(4,0.0001);
		covPMatrix.set(3,3,0);
		
		//covariances for lines
		Matrix covLMatrix = 
			LinearAlgebraFactory.getDiagMatrix(6,0.0001);
		
		//construct a line and a point
		Line_3D l1 = new Line_3D(0,0.5,2.5,7.5,0,0,covLMatrix);
		Point_3D p1 = new Point_3D(2.4,5.4,6.4,1,covPMatrix);
		
		//test if the reasoning of a plane is correct, using help function
		if(!helpPlane_3DPoint_3DLine_3DStatistics(p1,l1,0.2)) fail();
		
		//construct a line and a point
		Line_3D l2 = new Line_3D(-50,606,-705,3327,1998.5,2051,covLMatrix);
		Point_3D p2 = new Point_3D(1044,821.46,-211.42,9111,covPMatrix);
		
		//test if the reasoning of a plane is correct, using help function
		if(!helpPlane_3DPoint_3DLine_3DStatistics(p2,l2,0.3)) fail();
	}

	/**
	 * Test for void Plane_3D(Line_3D, Point_3D)
	 * Tests the reasoning of a plane out of a point and a line 
	 * with help of TestStatistics. 
	 */
	public void testPlane_3DLine_3DPoint_3D() {
		//coariances for plane
		Matrix covPMatrix = 
			LinearAlgebraFactory.getDiagMatrix(4,0.001);
		
		//covariances for lines
		Matrix covLMatrix = 
			LinearAlgebraFactory.getDiagMatrix(6,0.00001);
		
		//construct a line and a point
		Point_3D lP1 = new Point_3D(4,3,1,1,covPMatrix);
		Point_3D lP2 = new Point_3D(-4,4,1,1,covPMatrix);
		Line_3D l1 = new Line_3D(lP1,lP2);
		l1.setCov(covLMatrix);
		
		Point_3D p1 = new Point_3D(-2.14,-19,9.4,1,covPMatrix);
		
		//test if the reasoning of a plane is correct, using help function
		if(!helpPlane_3DLine_3DPoint_3DStatistics(l1,p1,0.25)) fail();
		
		//construct a line and a point
		Line_3D l2 = new Line_3D(-150,4506,1705,-1327,4998.5,2051,covLMatrix);
		Point_3D p2 = new Point_3D(4044,-921.46,-911.42,6111,covPMatrix);
		
		//test if the reasoning of a plane is correct, using help function
		if(!helpPlane_3DLine_3DPoint_3DStatistics(l2,p2,0.3)) fail();
	}

	/**
	 * Test for void Plane_3D(Point_3D, Point_3D, Point_3D)
	 * Tests the geometric reasoning of a plane out of three points
	 */
	public void testPlane_3DPoint_3DPoint_3DPoint_3D() {
		//covariances for points:
		Matrix covP = LinearAlgebraFactory.getDiagMatrix(4,0.01);
		covP.set(3,3,0);
		
		// Simple example of a plane out of three points:
		Point_3D pa = new Point_3D(-200342, 432, 111, covP);
		Point_3D pb = new Point_3D(-0.30342, 4432, 111,covP);
		Point_3D pc = new Point_3D(200, -2032, 111,covP);
		Plane_3D P= new Plane_3D(pa,pb,pc);
		P.normalizeEuclideanComparable();
		
		if (!TestStatistics.check(P.getV(), 
				LinearAlgebraFactory.getVector(0,0,-1,111), 1e-5))
			fail();
		
		//construct points
		Point_3D p1 = new Point_3D(2.002,-4.24,0,covP);
		Point_3D p2 = new Point_3D(0,5.3,-8.5,covP);
		Point_3D p3 = new Point_3D(-4.6,0,6.2,covP);
		
		//Test reasoning using help function
		if(!helpPlane_3DPoint_3DPoint_3DPoint_3DStatistics(p1,p2,p3,0.3)) fail();
		
		//construct points
		Point_3D p4 = new Point_3D(0,-6229,8922,covP);
		Point_3D p5 = new Point_3D(-2995,0,5291.2,covP);
		Point_3D p6 = new Point_3D(-4212,5833,0,covP);
		
		//Test reasoning using help function
		if(!helpPlane_3DPoint_3DPoint_3DPoint_3DStatistics(p4,p5,p6,0.3)) fail();
	}

	/**
	 * Test for void Plane_3D()
	 */
	public void testPlane_3D() {
		//construct plane
		Plane_3D p = new Plane_3D();
		
		//test if null.
		if(p == null) fail(); 
	}

	/**
	 * Test for copy()
	 * Copys plane and then changes original, 
	 * to secure that it is a real copy not only reference.
	 */
	public void testCopy() {
		//constructing plane
		Plane_3D p1 = new Plane_3D(1,1,1,1,LinearAlgebraFactory.getDiagMatrix(4, 0.04));
		
		//copy plane
		Plane_3D p2 = p1.copy();
		
		//change first plane
		p1.getV().set(0, 2);
		
		//nulling
		Vector diff = p1.getV().minus(p2.getV());
		Vector v = LinearAlgebraFactory.getVector(1, 1, 1,1);
		Vector diff2 = p2.getV().minus(v);
		Vector diff3 = (p2.getCov().minus(p1.getCov())).vec();
		
		//fail if null, because second point would be affected by changes after copy
		if(diff.getLength() == 0) fail();	
		//fail if not null, because copy would be wrong
		if(diff2.getLength() != 0) fail();
		//fail if not null, because copy of cov's would be wrong
		if(diff3.getLength() != 0) fail();
	}

	/**
	 * Test for Matrix Pi()
	 */
	public void testPi() {
		//construct plane
		Plane_3D p = new Plane_3D(1,2,3,4);
		
		//use Pi()
		Matrix sugrPi = p.Pi();
		
		//construct Pi "by foot"
		double[][] values = {	{4,0,0,-1},
								{0,4,0,-2},
								{0,0,4,-3},
								{0,-3,2,0},
								{3,0,-1,0},
								{-2,1,0,0}};
		Matrix myPi = LinearAlgebraFactory.getMatrix(values);		
		
		//test if values are equal
		if(!TestStatistics.check(sugrPi,myPi,0.00000001)) fail();
	}

	/**
	 * Test for Matrix Pi(Vector)
	 */
	public void testPiVector() {
		//construct Vector
		Vector v = LinearAlgebraFactory.getVector(1,2,3,4);
		
		//use Pi()
		Matrix sugrPi = Point_3D.Pi(v);
		
		//construct Pi "by foot"
		double[][] values = {	{4,0,0,-1},
								{0,4,0,-2},
								{0,0,4,-3},
								{0,-3,2,0},
								{3,0,-1,0},
								{-2,1,0,0}};
		Matrix myPi = LinearAlgebraFactory.getMatrix(values);		
		
		//test if values are equal
		if(!TestStatistics.check(sugrPi,myPi,0.0000001)) fail();
	}

	/**
	 * Test for Matrix PiDual()
	 */
	public void testPiDual() {
		//construct plane
		Plane_3D p = new Plane_3D(1,2,3,4);
		
		//use PiDual()
		Matrix sugrPiDual = p.PiDual();
		
		//construct Pi "by foot"
		// Be aware that the sign of all dual matrix are changed in
		// comparision of the diss. from S. Heuel, because we define
		// C with the other sign !
		double[][] values = {	{0,3,-2,0},
								{-3,0,1,0},
								{2,-1,0,0},
								{-4,0,0,1},
								{0,-4,0,2},
								{0,0,-4,3}};
		Matrix myPiDual = LinearAlgebraFactory.getMatrix(values);		
		
		//test if values are equal
		if(!TestStatistics.check(sugrPiDual,myPiDual,0.00000001)) fail();
	}

	/**
	 * Test for Matrix PiDual(Vector)
	 */
	public void testPiDualVector() {
		//construct point
		Vector v = LinearAlgebraFactory.getVector(1,2,3,4);
		
		//use PiDual()
		Matrix sugrPiDual = Plane_3D.PiDual(v);
		
		//construct Pi "by foot"
		// Be aware that the sign of all dual matrix are changed in
		// comparision of the diss. from S. Heuel, because we define
		// C with the other sign !
		double[][] values = {	{0,3,-2,0},
								{-3,0,1,0},
								{2,-1,0,0},
								{-4,0,0,1},
								{0,-4,0,2},
								{0,0,-4,3}};
		Matrix myPiDual = LinearAlgebraFactory.getMatrix(values);		
		
		//test if values are equal
		if(!TestStatistics.check(sugrPiDual,myPiDual,0.00000001)) fail();
	}

	/**
	 * Test for getDistanceToOrigin
	 * Test the distance to origin for one million different planes.
	 */
	public void testGetDistanceToOrigin() {
		//progress monitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
								"Calculating Distance","to origin.",0,1000000);
		
		//Test over 1 Million Planes
		outer:
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				//constructing planes
				Plane_3D p1 = new Plane_3D(i-100,(j+100)%1000,i,j-500.5);
				Plane_3D p2 = new Plane_3D((i+j)-1000,j-500,j,i-500.5);
		
				//using get Distance to Origin
				double dist1 = p1.getDistanceToOrigin();
				double dist2 = p2.getDistanceToOrigin();

				//calculate distance for myself
				double dist1Contr = Math.abs(p1.getV().get(3))/(p1.getHomogeneousPart().getLength());
				double dist2Contr = Math.abs(p2.getV().get(3))/(p2.getHomogeneousPart().getLength());
				
				//nulling: (needed because of exactness of double)
				double dist1val = Math.abs(dist1 - dist1Contr);
				double dist2val = Math.abs(dist2 - dist2Contr);
				
				//control values
				if(dist1val > 0.000000000001 || dist2val > 0.000000000001) {
					System.out.println("Dist1: " + dist1 + " \t\t Kontr: " + dist1Contr);
					System.out.println("Dist2: " + dist2 + " \t\t Kontr: " + dist2Contr);
					pm.close();
					fail();
				}
				
				if(pm.isCanceled()) break outer;
				else pm.setProgress(i*j);
			} 
		}
		pm.close();
		
		//special distances:
		Plane_3D p1 = new Plane_3D(1,2,1,0);
		Plane_3D p2 = new Plane_3D(0,0,0,1);
		
		//distance to origin:
		double dist1 = p1.getDistanceToOrigin();
		double dist2 = p2.getDistanceToOrigin();
		
		//control values:
		if(dist1 != 0) fail();
		if(dist2 != Double.POSITIVE_INFINITY) fail();
	}

}
