/*
 * Created on 03.06.2004
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * Test for Relation.
 * 
 * @author Achim Knigs
 */
public class RelationTest extends TestCase {

	/**
	 * Test for Relation(Entity, Entity, Class)
	 * Tests only for one example. Reason is given below.
	 */
	public void testRelationEntityEntityClass() {
		//entities with covs:
		double[][] covP1 = {{0.015,0.002,0.003},{0.002,0.016,0.004},{0.003,0.004,0.014}};
		double[][] covP2 = {{0.014,0.001,0},{0.001,0.015,0},{0,0,0}};
		
		Point_2D p2D = new Point_2D(1,2,
				LinearAlgebraFactory.getMatrix(covP1));
		Point_2D p2DStrich = new Point_2D(1,2,
				LinearAlgebraFactory.getMatrix(covP2));
		Point_2D p2DFail = new Point_2D(1.5,2,
				LinearAlgebraFactory.getMatrix(covP1));
		Point_2D noised = new Point_2D(1.0002,1.998,
				LinearAlgebraFactory.getMatrix(covP1));
		
		//Relation Objects, things only tested with one,
		//because when the jacobians are right, 
		//the calculations is allways correct, if it is 
		//for one case. Jacobians are tested in other places.
		Relation work = new Relation(p2D,p2DStrich,Equal.class);
		Relation fail = new Relation(p2DFail,p2DStrich,Equal.class);
		Relation noise = new Relation(p2D,noised,Equal.class);
		
		if(!work.decision())fail();
		if(fail.decision())fail();
		if(!noise.decision())fail();
	}

	/**
	 * Test for Relation(Entity, Entity, Class, boolean)
	 * mostly the same thing as above, but conditioning is switched on.
	 */
	public void testRelationEntityEntityClassboolean() {
		//entities with covs:
		double[][] covP1 = {{0.015,0.002,0.003},{0.002,0.016,0.004},{0.003,0.004,0.014}};
		double[][] covP2 = {{0.014,0.001,0.002},{0.001,0.015,0.003},{0.002,0.003,0.016}};
		
		Line_2D l2D = new Line_2D(1,2,1,
				LinearAlgebraFactory.getMatrix(covP1));
		Line_2D l2DStrich = new Line_2D(2,-1,-1,
				LinearAlgebraFactory.getMatrix(covP2));
		Line_2D l2DFail = new Line_2D(9,2,1,
				LinearAlgebraFactory.getMatrix(covP1));
		//a line with noise added, should work
		Line_2D noised = new Line_2D(2.007,-1.006,-0.997,
				LinearAlgebraFactory.getMatrix(covP2));
		
		//same as above
		Relation work = new Relation(l2D,l2DStrich,Orthogonal.class,true);
		Relation fail = new Relation(l2DFail,l2DStrich,Orthogonal.class,true);
		Relation workNoCon = new Relation(l2D,l2DStrich,Orthogonal.class,false);
		Relation failNoCon = new Relation(l2DFail,l2DStrich,Orthogonal.class,false);
		//try if test works when noise is added
		Relation noise = new Relation(l2D,noised,Orthogonal.class,true);

		if(!work.decision())fail(); //works on this example.. hm
		if(fail.decision())fail();
		if(!workNoCon.decision())fail();
		if(failNoCon.decision())fail();
		if(!noise.decision()) fail();
	}

	/**
	 * Test for Relation(BiRelationalProperty, Entity)
	 * mostly the same as above, but relationalproperty is 
	 * constructed before relation.
	 */
	public void testRelationBiRelationalPropertyEntity() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		//homogeneuspart is "secure" this time: :)
		double[][] covP2 = {	{0.014,0.001,0.002,0},
								{0.001,0.015,0.003,0},
								{0.002,0.003,0.016,0},
								{0,0,0,0}};
								
		Point_3D p3D = new Point_3D(1,2,3,
				LinearAlgebraFactory.getMatrix(covP1));
		//slightly modified values to simulate noise:
		Point_3D p3DStrich = new Point_3D(1.9,4.1,6.1,2,
				LinearAlgebraFactory.getMatrix(covP2));
		Point_3D p3DFail = new Point_3D(1.5,2,3,
				LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Equal eq = new Equal(p3DStrich);
		
		//then construct the two relations
		Relation work = new Relation(eq,p3D);
		Relation fail = new Relation(eq,p3DFail);
		
		if(!work.decision()) fail();
		if(fail.decision()) fail();
	}

	/**
	 * Test for Relation(BiRelationalProperty, Entity, boolean)
	 * Same as above, but condition is controlled explicitly here.
	 */
	public void testRelationBiRelationalPropertyEntityboolean() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		double[][] covP2 = {	{0.014,0.001,0.002,0.006},
								{0.001,0.015,0.003,0.007},
								{0.002,0.003,0.016,0.005},
								{0.006,0.007,0.005,0.017}};
		
		//slightly modified values to simulate noise:						
		Plane_3D plane = new Plane_3D(1.1,1.89,3.00451,1,
				LinearAlgebraFactory.getMatrix(covP1));
		Plane_3D planeStrich = new Plane_3D(2,4,6,2,
				LinearAlgebraFactory.getMatrix(covP2));
		Plane_3D planeFail = new Plane_3D(1.5,2,3,1,
				LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Parallel parallel = new Parallel(planeStrich,Plane_3D.class);
		
		//then construct the two relations
		Relation work = new Relation(parallel,plane,false);
		Relation fail = new Relation(parallel,planeFail,false);
		Relation workWithCon = new Relation(parallel,plane,true);
		Relation failWithCon = new Relation(parallel,planeFail,true);
		
		if(!work.decision()) fail();
		if(fail.decision()) fail();
		if(!workWithCon.decision()) fail();
		if(failWithCon.decision()) fail();
	}

	/**
	 * Test for decision.
	 * This only returns some value, calculated already in constructor.
	 * Is very similar to testing of the constructors:
	 */
	public void testDecision() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		double[][] covP2 = {	{0.014,0.001,0.002,0.006},
								{0.001,0.015,0.003,0.007},
								{0.002,0.003,0.016,0.005},
								{0.006,0.007,0.005,0.017}};
								
		Plane_3D plane = new Plane_3D(1,2,3,1,
				LinearAlgebraFactory.getMatrix(covP1));
		//plane that is orthogonal to first one, they are orthogonal, if scalarmult of homogenous part is 0
		//slightly modified values to simulate noise:
		Plane_3D planeStrich = new Plane_3D(3.2,3.02,-3.001,2,
				LinearAlgebraFactory.getMatrix(covP2));
		//plane that is not orthogonal to planestrich:
		Plane_3D planeFail = new Plane_3D(7,2,3,1,
				LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Orthogonal orthogonal = new Orthogonal(planeStrich,Plane_3D.class);
		
		//then construct the two relations
		Relation work = new Relation(orthogonal,plane,false);
		Relation fail = new Relation(orthogonal,planeFail,false);
		Relation workWithCon = new Relation(orthogonal,plane,true);
		Relation failWithCon = new Relation(orthogonal,planeFail,true);
		
		//decision test (this time):
		if(!work.decision()) fail();
		if(fail.decision()) fail();
		if(!workWithCon.decision()) fail();
		if(failWithCon.decision()) fail();
	}

	/**
	 * Test for testValue()
	 * This returns t.
	 * t = d^T * Sigma_dd^+ * d
	 */
	public void testTestValue() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		double[][] covP2 = {	{0.014,0.001,0.002,0.006},
								{0.001,0.015,0.003,0.007},
								{0.002,0.003,0.016,0.005},
								{0.006,0.007,0.005,0.017}};
								
		Point_3D point = new Point_3D(1,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		//plane that is orthogonal to first one, they are orthogonal, if scalarmult of homogenous part is 0
		//slightly modified values to simulate errors:
		Point_3D pointStrich = new Point_3D(3.2,3.02,-3.001,2,LinearAlgebraFactory.getMatrix(covP2));
		//plane that is not orthogonal to planestrich:
		Point_3D pointFail = new Point_3D(7,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Equal equal = new Equal(pointStrich,Point_3D.class);
		
		//then construct the two relations
		Relation work = new Relation(equal,point,false);
		Relation fail = new Relation(equal,pointFail,false);
		Relation workWithCon = new Relation(equal,point,true);
		Relation failWithCon = new Relation(equal,pointFail,true);
		
		//get Ts by hand:
		Vector d = work.getD();
		Matrix sigma = work.getSigma();
		double workT = d.scalarmult(sigma.pseudoInverse(equal.getDof()).mult(d));
		
		d = fail.getD();
		sigma = fail.getSigma();
		double failT = d.scalarmult(sigma.pseudoInverse(equal.getDof()).mult(d));
		
		d = workWithCon.getD();
		sigma = workWithCon.getSigma();
		double workWithConT = d.scalarmult(sigma.invert().mult(d));
		
		d = failWithCon.getD();
		sigma = failWithCon.getSigma();
		double failWithConT = d.scalarmult(sigma.invert().mult(d));
		
		//compare Ts:
		if(workT != work.testValue()) fail();
		if(failT != fail.testValue()) fail();
		if(workWithConT != workWithCon.testValue()) fail();
		if(failWithConT != failWithCon.testValue()) fail();
	}

	/**
	 * Test for getChiSquareDistributionValue()
	 * tests alpha of 95, 99, and 90
	 * for all used dof's
	 * 
	 * Differences to Matlab:
	 *  all getting better from dof1 to dof4
	 *  for alpha = 0.95 dof1 differs 0.09, dof4 only 0.02
	 *  for alpha = 0.90 dof1 differs 0.06, dof4 0.03
	 *  for alpha = 0.99 dof1 differs 0.04, dof2 0.01,
	 *  							dof2 and dof3 0.02
	 * 
	 */
	public void testGetChiSquareDistributionValue() {
		double[] zeros = new double[12];
		
		//dof 1:
		zeros[0] = Math.abs(
				Relation.getChiSquareDistributionValue(0.95,1)
				- 3.8415);
		zeros[1] = Math.abs(
				Relation.getChiSquareDistributionValue(0.90,1)
				- 2.7055);
		zeros[2] = Math.abs(
				Relation.getChiSquareDistributionValue(0.99,1)
				- 6.6349);
		
		//dof 2:
		zeros[3] = Math.abs(
				Relation.getChiSquareDistributionValue(0.95,2)
				- 5.9915);
		zeros[4] = Math.abs(
				Relation.getChiSquareDistributionValue(0.90,2)
				- 4.6052);
		zeros[5] = Math.abs(
				Relation.getChiSquareDistributionValue(0.99,2)
				- 9.2103);
		
		//dof 3:
		zeros[6] = Math.abs(
				Relation.getChiSquareDistributionValue(0.95,3)
				- 7.8147);
		zeros[7] = Math.abs(
				Relation.getChiSquareDistributionValue(0.90,3)
				- 6.2514);
		zeros[8] = Math.abs(
				Relation.getChiSquareDistributionValue(0.99,3)
				- 11.3449);
		
		//dof 4:
		zeros[9] = Math.abs(
				Relation.getChiSquareDistributionValue(0.95,4)
				- 9.4877);
		zeros[10] = Math.abs(
				Relation.getChiSquareDistributionValue(0.90,4)
				- 7.7794);
		zeros[11] = Math.abs(
				Relation.getChiSquareDistributionValue(0.99,4)
				- 13.2767);
		
		for(int i = 0; i < zeros.length; i++){
			if(zeros[i] > 0.1) fail();
		}
	}

	/**
	 * getter for alpha, used in Relation
	 */
	public void testGetAlpha() {
		//value hardcoded in Relation.java
		double alpha  = 0.95;
		
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.01);
		cov.set(2,2,0);
		
		//construct relation
		Point_2D p = new Point_2D(1,2,cov);
		Point_2D pStrich = new Point_2D(2,4,cov);
		
		Relation rel = new Relation(p,pStrich,Equal.class);
		
		if(rel.getAlpha() != alpha) fail();
	}

	/**
	 * setter for alpha.
	 * Test tests if setting alpha shows effect on calculation.
	 */
	public void testSetAlpha() {
		//entities with covs:
		double[][] covP1 = {{0.015,0.002,0.003},{0.002,0.016,0.004},{0.003,0.004,0.014}};
		double[][] covP2 = {{0.014,0.001,0},{0.001,0.015,0},{0,0,0}};
		
		Point_2D p2D = new Point_2D(1,2,LinearAlgebraFactory.getMatrix(covP1));
		//this point works for alpha = 0.99, but not for alpha = 0.95,
		//so if test fails, alpha seems to be ignored.
		Point_2D p2DStrich = new Point_2D(1.5,2,LinearAlgebraFactory.getMatrix(covP2));
		
		//Relation Objects, things only tested with one,
		//because when the jacobians are right, 
		//the calculations is allways correct, if it is 
		//for one case. Jacobians are tested in other places.
		Relation work = new Relation(p2D,p2DStrich,Equal.class);
		work.setAlpha(0.99);
		Relation fail = new Relation(p2D,p2DStrich,Equal.class);
		fail.setAlpha(0.95);
		
		if(!work.decision())fail();
		if(fail.decision()) fail();
	}

	/**
	 * recalculates the test!
	 * But conditioning itself is tested in estimation tests!
	 * here only tests if flag is set properly.
	 */
	public void testSetCondition() {
		//entities with covs:
		double[][] covP1 = {{0.015,0.002,0.003},{0.002,0.016,0.004},{0.003,0.004,0.014}};
		double[][] covP2 = {{0.014,0.001,0.002},{0.001,0.015,0.003},{0.002,0.003,0.016}};
		
		Line_2D l2D = new Line_2D(2000000,2000000,2000000,LinearAlgebraFactory.getMatrix(covP1));
		Line_2D l2DStrich = new Line_2D(2000000,-2000000,-2000000,LinearAlgebraFactory.getMatrix(covP2));
		
		//same as above
		Relation rel = new Relation(l2D,l2DStrich,Orthogonal.class,false);

		if(!rel.decision())fail();
		
		rel.setCondition(true);
		if(!rel.getCondition())fail();
	}
	
	/**
	 * getter for condition flag
	 */
	public void testGetCondition() {
		//entities with covs:
		double[][] covP1 = {{0.015,0.002,0.003},{0.002,0.016,0.004},{0.003,0.004,0.014}};
		double[][] covP2 = {{0.014,0.001,0.002},{0.001,0.015,0.003},{0.002,0.003,0.016}};
		
		Line_2D l2D = new Line_2D(1,2,1,LinearAlgebraFactory.getMatrix(covP1));
		Line_2D l2DStrich = new Line_2D(2,-1,-1,LinearAlgebraFactory.getMatrix(covP2));
		
		//same as above
		Relation rel = new Relation(l2D,l2DStrich,Orthogonal.class,true);
		Relation relNoCon = new Relation(l2D,l2DStrich,Orthogonal.class,false);

		if(!rel.getCondition())fail(); //works on this example.. hm
		if(relNoCon.getCondition())fail();

	}

	/**
	 * gets the test threshold, that is chi
	 *
	 */
	public void testTestThreshold() {
		Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(3,0.01);
		Matrix cov4 = LinearAlgebraFactory.getDiagMatrix(4,0.01);
		Matrix cov6 = LinearAlgebraFactory.getDiagMatrix(6,0.01);
		
		//construct some entities:
		Point_2D p2D = new Point_2D(1,2,3,cov3);
		Line_2D l2D = new Line_2D(1,2,3,cov3);
		Point_3D p3D = new Point_3D(1,2,3,4,cov4);
		Plane_3D plane = new Plane_3D(1,2,3,4,cov4);
		Line_3D l3D = new Line_3D(1,2,3,4,5,6,cov6);
		
		//my relation ;)
		Relation rel;
		
		//zeros:
		double[] zeros = new double[20];
		
		//chi2-values for alpha = 0.95:
		double dof1 = 3.8415;
		double dof2 = 5.9915;
		double dof3 = 7.8147;
		double dof4 = 9.4877;
		
		//now test all possible relations, if they have right 
		//testThreshold:
		
		//Incidents:
		rel = new Relation(p2D,l2D,Incident.class);
		zeros[0] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(p3D,plane,Incident.class);
		zeros[1] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(p3D,l3D,Incident.class);
		zeros[2] = Math.abs(
				rel.testThreshold() - dof2);
		
		rel = new Relation(l3D,l3D,Incident.class);
		zeros[3] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(l3D,plane,Incident.class);
		zeros[4] = Math.abs(
				rel.testThreshold() - dof2);
		
		//Equal
		rel = new Relation(p2D,p2D,Equal.class);
		zeros[5] = Math.abs(
				rel.testThreshold() - dof2);
		
		rel = new Relation(l2D,l2D,Equal.class);
		zeros[6] = Math.abs(
				rel.testThreshold() - dof2);
		
		rel = new Relation(p3D,p3D,Equal.class);
		zeros[7] = Math.abs(
				rel.testThreshold() - dof3);
		
		rel = new Relation(l3D,l3D,Equal.class);
		zeros[8] = Math.abs(
				rel.testThreshold() - dof4);
		
		rel = new Relation(plane,plane,Equal.class);
		zeros[9] = Math.abs(
				rel.testThreshold() - dof3);
		
		//Parallel
		rel = new Relation(l2D,l2D,Parallel.class);
		zeros[10] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(l3D,l3D,Parallel.class);
		zeros[11] = Math.abs(
				rel.testThreshold() - dof2);
		
		rel = new Relation(plane,plane,Parallel.class);
		zeros[12] = Math.abs(
				rel.testThreshold() - dof2);
		
		rel = new Relation(plane,l3D,Parallel.class);
		zeros[13] = Math.abs(
				rel.testThreshold() - dof1);
		
		//Orthogonal
		rel = new Relation(l2D,l2D,Orthogonal.class);
		zeros[14] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(l3D,l3D,Orthogonal.class);
		zeros[15] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(plane,plane,Orthogonal.class);
		zeros[16] = Math.abs(
				rel.testThreshold() - dof1);
		
		rel = new Relation(plane,l3D,Orthogonal.class);
		zeros[17] = Math.abs(
				rel.testThreshold() - dof2);
		
		//now testing, if new value is proper for different alpha:
		rel.setAlpha(0.90);
		zeros[18] = Math.abs(
				rel.testThreshold() - 4.6052);
		
		rel.setAlpha(0.99);
		zeros[19] = Math.abs(
				rel.testThreshold() - 9.2103);
		
		for(int i = 0; i < 20; i++){
			if(zeros[i] > 0.1) fail();
		}
	}

	/**
	 * gets D. That is the unmodified result of the multiplication
	 * with the jacobians (if near zero relation is true)
	 */
	public void testGetD() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		double[][] covP2 = {	{0.014,0.001,0.002,0.006},
								{0.001,0.015,0.003,0.007},
								{0.002,0.003,0.016,0.005},
								{0.006,0.007,0.005,0.017}};
								
		Plane_3D plane = new Plane_3D(1,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		//plane that is orthogonal to first one, they are orthogonal, if scalarmult of homogenous part is 0
		//slightly modified values to simulate errors:
		Plane_3D planeStrich = new Plane_3D(3.2,3.02,-3.001,2,LinearAlgebraFactory.getMatrix(covP2));
		//plane that is not orthogonal to planestrich:
		Plane_3D planeFail = new Plane_3D(9,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Orthogonal orthogonal = new Orthogonal(plane,Plane_3D.class);
		
		//then construct the two relations
		Relation work = new Relation(orthogonal,planeStrich);
		Relation fail = new Relation(orthogonal,planeFail);
		 
		//get d's by hand
		Matrix jacobian = orthogonal.getJacobian();
		
		Vector workD = jacobian.mult(planeStrich.getV());
		Vector failD = jacobian.mult(planeFail.getV());
		
		//compare to d's from sugr:
		if(workD.minus(work.getD()).getLength() != 0) fail();
		if(failD.minus(fail.getD()).getLength() != 0) fail();
		if(fail.decision()) fail();
		if(!work.decision()) fail();
	}

	/**
	 * gets sigma_dd, calculated in constructor
	 */
	public void testGetSigma() {
		//entities with covs:
		double[][] covP1 = {	{0.015,0.002,0.003,0.005},
								{0.002,0.016,0.004,0.006},
								{0.003,0.004,0.014,0.007},
								{0.005,0.006,0.007,0.017}};
		double[][] covP2 = {	{0.014,0.001,0.002,0.006},
								{0.001,0.015,0.003,0.007},
								{0.002,0.003,0.016,0.005},
								{0.006,0.007,0.005,0.017}};
								
		Plane_3D plane = new Plane_3D(1,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		//plane that is orthogonal to first one, they are orthogonal, if scalarmult of homogenous part is 0
		//slightly modified values to simulate errors:
		Plane_3D planeStrich = new Plane_3D(3.2,3.02,-3.001,2,LinearAlgebraFactory.getMatrix(covP2));
		//plane that is not orthogonal to planestrich:
		Plane_3D planeFail = new Plane_3D(7,2,3,1,LinearAlgebraFactory.getMatrix(covP1));
		
		//construct relation:
		Orthogonal orthogonal = new Orthogonal(planeStrich,Plane_3D.class);
		
		//then construct the two relations
		Relation work = new Relation(orthogonal,plane);
		Relation fail = new Relation(orthogonal,planeFail);
		
		//get d's by hand
		Matrix A = orthogonal.getJacobian();
		Matrix B = plane.getJacobianOfRelation(Orthogonal.class,Plane_3D.class);
		
		Matrix workSigma = A.mult(plane.getCov().mult(A.trans())).plus(
				B.mult(planeStrich.getCov().mult(B.trans())));
		
		B = planeFail.getJacobianOfRelation(Orthogonal.class,Plane_3D.class);
		Matrix failSigma = A.mult(planeFail.getCov().mult(A.trans())).plus(
				B.mult(planeStrich.getCov().mult(B.trans())));
		
		//compare to d's from sugr:
		if(workSigma.minus(work.getSigma()).vec().getLength() != 0) fail();
		if(failSigma.minus(fail.getSigma()).vec().getLength() != 0) fail();
	}
	

	/**
	 * Number of iterations used to generate gaussians in each test
	 */
	int MAXITER = 100000;
	
	/**
	 * Tests if tests are calculated correctly.
	 * Doesn't test ALL possible combinations, because
	 * the test relys on the jacobians.
	 */
	public void testTests(){
		
		//covs
		double[][] cov2D = {{0.015,0.001,0.001},{0.001,0.015,0.001},{0.001,0.001,0.015}};
		double[][] cov3D = {{0.14,0.001,0.002,0.003},
							{0.001,0.16,0.004,0.005},
							{0.002,0.004,0.14,0.006},
							{0.003,0.005,0.006,0.13}};
		double[][] covL = {	{0.15148,0.002,0.002,0.002,0.002,0.002},
							{0.002,0.14623,0.002,0.002,0.002,0.002},
							{0.002,0.002,0.15452,0.002,0.002,0.002},
							{0.002,0.002,0.002,0.16044,0.002,0.002},
							{0.002,0.002,0.002,0.002,0.14024,0.002},
							{0.002,0.002,0.002,0.002,0.002,0.13123}};
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
			"Testing tests.","",0,MAXITER*15);

		int failures = 0;
		
		//Point 2D Equal
		Point_2D p2DEq = new Point_2D(245,-234,
				LinearAlgebraFactory.getMatrix(cov2D));
		Point_2D p2DEqStrich = new Point_2D(245,-234,
				LinearAlgebraFactory.getMatrix(cov2D));
		
		if(!helpTests(p2DEq,p2DEqStrich,Equal.class,0,pm,
				"Point_2D Equal")){
			failures++;
		}
		
		//Point 2D Line 2D Incitent
		Point_2D p2DL2DIncP2D = new Point_2D(0,1,0,
				LinearAlgebraFactory.getMatrix(cov2D));
		Line_2D p2DL2DIncL2D = new Line_2D(0,0,-1,
				LinearAlgebraFactory.getMatrix(cov2D));

		if(!helpTests(p2DL2DIncP2D,p2DL2DIncL2D,Incident.class,MAXITER,pm,
				"Point_2D Line_2D Incident")){
			failures++;
		}
		
		//Line 2D Equal
		Line_2D l2DEq = new Line_2D(518,-58.1302,7.720,
				LinearAlgebraFactory.getMatrix(cov2D));
		Line_2D l2DEqStrich = new Line_2D(518.006,-58.13,7.720,
				LinearAlgebraFactory.getMatrix(cov2D));

		if(!helpTests(l2DEq,l2DEqStrich,Equal.class,MAXITER*2,pm,
				"Line_2D Equal")){
			failures++;
		}
		
		//Line 2D Parallel
		Line_2D l2DPar = new Line_2D(20,35,5,
				LinearAlgebraFactory.getMatrix(cov2D));
		Line_2D l2DParStrich = new Line_2D(20,35,9,
				LinearAlgebraFactory.getMatrix(cov2D));
		
		if(!helpTests(l2DPar,l2DParStrich,Parallel.class,MAXITER*3,pm,
				"Line_2D Parallel")){
			failures++;
		}
		
		//Line 2D Ortthogonal
		Line_2D l2DOrtho = new Line_2D(-2,1,1,
				LinearAlgebraFactory.getMatrix(cov2D));
		Line_2D l2DOrthoStrich = new Line_2D(1,2,3,
				LinearAlgebraFactory.getMatrix(cov2D));
		
		if(!helpTests(l2DOrtho,l2DOrthoStrich,Orthogonal.class,MAXITER*4,pm,
				"Line_2D Orthogonal")){
			failures++;
		}
		
		//Point 3D Plane 3D Incident
		Point_3D p3DPlaneIncP3D = new Point_3D(2,-2,2,-2,
				LinearAlgebraFactory.getMatrix(cov3D));
		Plane_3D p3DPlaneIncPlane = new Plane_3D(2,2,2,2,
				LinearAlgebraFactory.getMatrix(cov3D));
		if(!helpTests(p3DPlaneIncP3D,p3DPlaneIncPlane,Incident.class,MAXITER*5,pm,
				"Point_3D Plane_3D Incident")){
			failures++;
		}
		
		//Point 3D Equal
		Point_3D p3DEq = new Point_3D(-1006,421,6442.43,1,
				LinearAlgebraFactory.getMatrix(cov3D));
		Point_3D p3DEqStrich = new Point_3D(-1005.9991,421.002,6442.429,1.004,
				LinearAlgebraFactory.getMatrix(cov3D));
		if(!helpTests(p3DEq,p3DEqStrich,Equal.class,MAXITER*6,pm,
				"Point_3D Equal")){
			failures++;
		}
		
		//Plane Equall
		Plane_3D planeEq = new Plane_3D(10.621,21,-12,53,
				LinearAlgebraFactory.getMatrix(cov3D));
		if(!helpTests(planeEq,(Plane_3D)planeEq.clone(),Equal.class,MAXITER*7,pm,
				"Plane_3D Equal")){
			failures++;
		}
		
		//Plane Plane parallel
		Plane_3D planePlaneParP1 = new Plane_3D(-15,3,8,9.07,
				LinearAlgebraFactory.getMatrix(cov3D));
		Plane_3D planePlaneParP2 = new Plane_3D(-15,3,8,33,
				LinearAlgebraFactory.getMatrix(cov3D));
		if(!helpTests(planePlaneParP1,planePlaneParP2,Parallel.class,MAXITER*8,pm,
				"Plane_3D Plane_3D Parallel")){
			failures++;
		}
		
		//Plane Line_3D Parallel
		Plane_3D planeL3DParPlane = new Plane_3D(2,0,-2,8,
				LinearAlgebraFactory.getMatrix(cov3D));
		Line_3D planeL3DParL3D = new Line_3D(2,0,2,76,43,97,
				LinearAlgebraFactory.getMatrix(covL));
		if(!helpTests(planeL3DParPlane,planeL3DParL3D,Parallel.class,MAXITER*9,pm,
				"Plane_3D Line_3D Parallel")){
			failures++;
		}
		
		//Plane Plane Orthogonal
		Plane_3D planePlaneOrthoP1 = new Plane_3D(-4,6,2,.63,
				LinearAlgebraFactory.getMatrix(cov3D));
		Plane_3D planePlaneOrthoP2 = new Plane_3D(-2,-3,5,89,
				LinearAlgebraFactory.getMatrix(cov3D));
		if(!helpTests(planePlaneOrthoP1,planePlaneOrthoP2,
				Orthogonal.class,MAXITER*10,pm,
				"Plane_3D Plane_3D Orthogonal")){
			failures++;
		}
		
		//Plane Line_3D Orthogonal
		Plane_3D planeL3DOrthogonalP = new Plane_3D(-3,-7.5,3,99,
				LinearAlgebraFactory.getMatrix(cov3D));
		Line_3D planeL3DOrthogonalL = new Line_3D(2,5,-2,5,23,33,
				LinearAlgebraFactory.getMatrix(covL));
		if(!helpTests(planeL3DOrthogonalP,planeL3DOrthogonalL,
				Orthogonal.class,MAXITER*11,pm,
				"Plane_3D Line_3D Orthogonal")){
			failures++;
		}
		
		//Line_3D Equal
		//Line 3D Line 3D Jacobian kaputt?
		Line_3D l3DEq = new Line_3D(-10,10,-5,-200,-150,100,
				LinearAlgebraFactory.getMatrix(covL));
		Line_3D l3DEqStrich = new Line_3D(-10,10,-5,-200,-150,100,
				LinearAlgebraFactory.getMatrix(covL));		

		if(!helpTests(l3DEq,l3DEqStrich,Equal.class,
				MAXITER*12,pm,
				"Line_3D Equal")){
			failures++;
		}
		
		//Line_3D Line_3D Parallel
		Line_3D l3DL3DParL1 = new Line_3D(2,2,2,76,-33,-77,
				LinearAlgebraFactory.getMatrix(covL));
		Line_3D l3DL3DParL2 = new Line_3D(2,2,2,45,-8.9,43,
				LinearAlgebraFactory.getMatrix(covL));
		if(!helpTests(l3DL3DParL1,l3DL3DParL2,Parallel.class,
				MAXITER*13,pm,"Line_3D Line_3D Parallel")){
			failures++;
		}
		
		//Line_3D Line_3D Orthogonal
		Line_3D l3DL3DOrthoL1 = new Line_3D(2,-2,0,63,-63,-2,
				LinearAlgebraFactory.getMatrix(covL));
		Line_3D l3DL3DOrthoL2 = new Line_3D(2,2,0,52,-65,-6,
				LinearAlgebraFactory.getMatrix(covL));
		if(!helpTests(l3DL3DOrthoL1,l3DL3DOrthoL2,Orthogonal.class,
				MAXITER*14,pm,"Line_3D Line_3D Orthogonal")){
			failures++;
		}
		
		pm.close();
		
		System.out.println("Number of Tests failed: " + failures +
				" of 14 Tests.");
		if(failures > 0) fail();
	}
	
	public boolean helpTests(Entity first, Entity second, Class relation,int start, 
			TestTools.ProgressMonitorWithTaskbar pm,String name){
		pm.setNote(name);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(first.getV(),first.getCov());
		RandomVector vl2 = new RandomVector(second.getV(),second.getCov());
		
		//a temp point
		Relation rel = new Relation(first,second,relation);
		
		if(!rel.decision()){
			System.out.println("Test " + name + " failed in SUGR, no " +
					"further test possible, please change values.");
			System.out.println("'d' was: " + rel.getD());
			return false;
		}
		
		int numPassed = 0;
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			first.setV(vl1.getNextRandom());
			second.setV(vl2.getNextRandom());
			
			//calculate point and add observation
			rel = new Relation(first,second,relation);
			
			if(rel.decision()) numPassed++;
						
			if(pm.isCanceled()) break;
			pm.setProgress(start+i);
		}
		
		//have 95% passed?
		double relNumPassed = (double)numPassed/(double)MAXITER;
		double zeroOrLess = rel.getAlpha() - relNumPassed;
		boolean retval = (zeroOrLess < 0.2);
		
		if(zeroOrLess > 0) {
			System.out.println("'" + name 
					+ "' differed a little, relative number of tests hold: " 
					+ (double)numPassed/(double)MAXITER 
					+ " alpha was: " + rel.getAlpha());
		}
		return retval;
	}
}
