/*
 * Created on 14.03.2004
 *
 * Test methods used for every entity.
 */
package sugr.test;

import javax.swing.*;
import java.awt.*;
import sugr.*;

/**
 * Class that contaions some functions often used to test
 * sugr classes.
 * 
 * @author Achim Knigs
 */
public class TestTools {

	/**
	 * Function that test's if the normalization of a point still is
	 * correct when uncertainity is used 
	 * @param test - the entity to be tested
	 * @param iterations - the number of iterations (should be at least >=100,000)
     * @param prec - the relative precision with which the results must match 
	 * @return - true if test was successful.
	 */
	public static boolean helpNormalizeEuclideanStatistics(Entity test, int iterations, double prec){
		//generate Statistics Object
		TestStatistics stat = new TestStatistics(test.getDimension());
		
		//One RandomVector, because only one random value is used
		RandomVector vl1 = new RandomVector(test.getV(),test.getCov());
		
		//a temporary entity
		Entity norm;
		
		//ProgressMonitor:
		ProgressMonitorWithTaskbar pm = new ProgressMonitorWithTaskbar("Test statistics part of normalize euclidean","",0,iterations);
		
		//iterate
		for(int i = 0; i < iterations; i++)
		{
			//get a independent copy of the entity (can't use constructor here) 
			norm = (Entity) test.clone();
			
			//set the vector to some random value 
			norm.setV(vl1.getNextRandom());
			
			//use normalizeEuclidean:
			norm.normalizeEuclidean();
			
			//add the observation
			stat.addObservation(norm);
			
			//update progress window
			if(pm.isCanceled()) break;
			pm.setProgress(i);	
		}
		pm.close();

		//let sugr calculate the normalization: 
		norm = (Entity) test.clone();
		norm.normalizeEuclidean();
		
		//test the results:
		boolean retval = TestStatistics.check(norm.getV(), stat.getMue(), 
											  norm.getCov(), stat.getSigma(), prec);
		if(!retval){
			System.out.println("Normalize Euclidean of " + test.getClass().getName() + " failed.");
			System.out.println("SUGR Values: " + norm);
			System.out.println("Calculated Mue: " + stat.getMue());
			System.out.println("Calculated Sigma: " + stat.getSigma());
		}
		return retval;
	}
	
	/**
	 * Function that test's if the normalization of a point still is
	 * correct when uncertainity is used 
	 * @param test - the point to be tested
	 * @param iterations - the number of iterations (should be at least >=100,000)
	 * @return - true if test was successful.
	 */
	public static boolean helpNormalizeEuclideanStatistics(Entity test, int iterations){
		return helpNormalizeEuclideanStatistics(test,iterations,0.1);
	}

	/**
	 * Function that test's if the normalization of a point still is
	 * correct when uncertainity is used 
	 * @param test - the entity to be tested
	 * @return - true if test was successful.
	 */
	public static boolean helpNormalizeEuclideanStatistics(Entity test){
		return helpNormalizeEuclideanStatistics(test,1000000,0.1);
	}
	
	/**
	 * Function that test's if the normalization of a point still is
	 * correct when uncertainity is used 
	 * @param test - the entity to be tested
	 * @param prec - the relative precision with which the results must match 
	 * @return - true if test was successful.
	 */
	public static boolean helpNormalizeEuclideanStatistics(Entity test, double prec){
		return helpNormalizeEuclideanStatistics(test,1000000,prec);
	}
	
	
	/**
	 * Method to test all unallowed possibilities of getJacobianOfRelatoin
	 * returns false if one of the unallowed combinations doesn't return an error.
	 * @param allowed - two dimensional array of classes which are all allowed combinations
	 * 					for that entity
	 * @param ent - an entity of the class to be tested
	 * @return - true if all unallowed combinations fail, false if one doesn't.
	 */
	public static boolean helpJacobianOfRelations(Class[][] allowed, Entity ent) {
		//All possibilities
		Class[][] tests = {	
			{Equal.class,Point_2D.class},
			{Equal.class,Line_2D.class},
			{Equal.class,Line_3D.class},
			{Equal.class,Point_3D.class},
			{Equal.class,Plane_3D.class},
			{Incident.class,Point_2D.class},
			{Incident.class,Line_2D.class},
			{Incident.class,Point_3D.class},
			{Incident.class,Plane_3D.class},														
			{Incident.class,Line_3D.class},
			{Orthogonal.class,Point_2D.class},
			{Orthogonal.class,Line_2D.class},
			{Orthogonal.class,Point_3D.class},
			{Orthogonal.class,Plane_3D.class},
			{Orthogonal.class,Line_3D.class},
			{Parallel.class,Point_2D.class},
			{Parallel.class,Line_2D.class},
			{Parallel.class,Point_3D.class},
			{Parallel.class,Plane_3D.class},
			{Parallel.class,Line_3D.class}};
		//Vector to save the unallowed combinations
		java.util.Vector notallowed = new java.util.Vector();

		//search the unallowed combinations
		int nr = 0;
		for(int i = 0; i < tests.length; i++) {
			boolean inA = false;
			for(int j = 0; j < allowed.length; j++) {
				if((tests[i][0] == allowed[j][0]) &&
					(tests[i][1] == allowed[j][1])) {
					inA = true;	
				}
			}
			if(!inA){
				notallowed.add(tests[i]);
				nr++;
			}
		}
		
		//try every unallowed combination,
		//if it works, return false, because it should throw an error.
		for(int i = 0; i < notallowed.size(); i++) {
			try {
				Class[] test = (Class[]) notallowed.get(i);
				ent.getJacobianOfRelation(test[0],test[1]);
				System.out.println("Classes which didn't fail when using getJacobianOfRelation: ");
				System.out.println(test[0] + " and " + test[1]);
				return false;
			}
			catch(IllegalArgumentException e) {}
		}
		return true;
	}


	/**
	 * Method to test all unallowed possibilities of getDofOfRelatoin
	 * returns false if one of the unallowed combinations doesn't return an error.
	 * @param allowed - two dimensional array of classes which are all allowed combinations
	 * 					for that entity
	 * @param ent - an entity of the class to be tested
	 * @return - true if all unallowed combinations fail, false if one doesn't.
	 */
	public static boolean helpDofOfRelations(Class[][] allowed, Entity ent) {
		//All possibilities
		Class[][] tests = {	
			{Equal.class,Point_2D.class},
			{Equal.class,Line_2D.class},
			{Equal.class,Line_3D.class},
			{Equal.class,Point_3D.class},
			{Equal.class,Plane_3D.class},
			{Incident.class,Point_2D.class},
			{Incident.class,Line_2D.class},
			{Incident.class,Point_3D.class},
			{Incident.class,Plane_3D.class},														
			{Incident.class,Line_3D.class},
			{Orthogonal.class,Point_2D.class},
			{Orthogonal.class,Line_2D.class},
			{Orthogonal.class,Point_3D.class},
			{Orthogonal.class,Plane_3D.class},
			{Orthogonal.class,Line_3D.class},
			{Parallel.class,Point_2D.class},
			{Parallel.class,Line_2D.class},
			{Parallel.class,Point_3D.class},
			{Parallel.class,Plane_3D.class},
			{Parallel.class,Line_3D.class}};
		
		//Vector to save the unallowed combinations
		java.util.Vector notallowed = new java.util.Vector();

		//search the unallowed combinations
		int nr = 0;
		for(int i = 0; i < tests.length; i++) {
			boolean inA = false;
			for(int j = 0; j < allowed.length; j++) {
				if((tests[i][0] == allowed[j][0]) &&
					(tests[i][1] == allowed[j][1])) {
					inA = true;	
				}
			}
			if(!inA) {
				notallowed.add(tests[i]); 
				nr++;
			}
		}
	
		//try every unallowed combination,
		//if it works, return false, because it should throw an error.
		for(int i = 0; i < notallowed.size(); i++) {
			try {
				Class[] test = (Class[]) notallowed.get(i);
				ent.getDofOfRelation(test[0],test[1]);
				System.out.println("Classes which didn't fail when using getDofOfRelation: ");
				System.out.println(test[0] + " and " + test[1]);				
				return false;
			}
			catch(IllegalArgumentException e) {}
		}
		return true;
	}


	public static class ProgressMonitorWithTaskbar extends ProgressMonitor {
		JFrame window;
		private ProgressMonitorWithTaskbar(JFrame w,String name, String remark, int begining, int end) {
			super(w,name,remark,begining,end);
			this.window = w;
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			window.setLocation((screen.width-100)/2,(screen.height-80)/2);
			window.setUndecorated(true);
			window.setResizable(false);
			window.setVisible(true);
		}
		public ProgressMonitorWithTaskbar(String name,String remark, int begining, int end) {
			this(new JFrame("Progress Monitor"),name,remark,begining,end);
		}
		public void close() {
			super.close();
			window.dispose();
		}
	}
}
